@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (C) 2014-2022 GEM Foundation
+# Copyright (C) 2014-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -292,6 +292,7 @@ def gen_slices(start, stop, blocksize):
     >>> list(gen_slices(1, 6, 2))
     [slice(1, 3, None), slice(3, 5, None), slice(5, 6, None)]
     """
+    blocksize = int(blocksize)
     assert start <= stop, (start, stop)
     assert blocksize > 0, blocksize
     while True:
@@ -944,8 +945,6 @@ def multi_index(shape, axis=None):
     (slice(None, None, None), 1, 1)
     (slice(None, None, None), 1, 2)
     """
-    if any(s >= TWO16 for s in shape):
-        raise ValueError('Shape too big: ' + str(shape))
     ranges = (range(s) for s in shape)
     if axis is None:
         yield from itertools.product(*ranges)
@@ -955,6 +954,7 @@ def multi_index(shape, axis=None):
         yield tuple(lst)
 
 
+# NB: the fast_agg functions are usually faster than pandas
 def fast_agg(indices, values=None, axis=0, factor=None, M=None):
     """
     :param indices: N indices in the range 0 ... M - 1 with M < N
@@ -989,6 +989,7 @@ def fast_agg(indices, values=None, axis=0, factor=None, M=None):
     return res
 
 
+# NB: the fast_agg functions are usually faster than pandas
 def fast_agg2(tags, values=None, axis=0):
     """
     :param tags: N non-unique tags out of M
@@ -1009,6 +1010,7 @@ def fast_agg2(tags, values=None, axis=0):
     return uniq, fast_agg(indices, values, axis)
 
 
+# NB: the fast_agg functions are usually faster than pandas
 def fast_agg3(structured_array, kfield, vfields=None, factor=None):
     """
     Aggregate a structured array with a key field (the kfield)
@@ -1040,36 +1042,6 @@ def fast_agg3(structured_array, kfield, vfields=None, factor=None):
     return res
 
 
-# this is fast
-def kmean(structured_array, kfield, uniq_indices_counts=()):
-    """
-    Given a structured array of N elements with a discrete kfield with
-    K <= N unique values, returns a structured array of K elements
-    obtained by averaging the values associated to the kfield.
-    """
-    allnames = structured_array.dtype.names
-    assert kfield in allnames, kfield
-    if uniq_indices_counts:
-        uniq, indices, counts = uniq_indices_counts
-    else:
-        uniq, indices, counts = numpy.unique(
-            structured_array[kfield], return_inverse=True, return_counts=True)
-    dic = {}
-    dtlist = []
-    for name in allnames:
-        if name == kfield:
-            dic[kfield] = uniq
-        else:
-            values = structured_array[name]
-            dic[name] = fast_agg(indices, values) / (
-                counts if len(values.shape) == 1 else counts.reshape(-1, 1))
-        dtlist.append((name, structured_array.dtype[name]))
-    res = numpy.zeros(len(uniq), dtlist)
-    for name in dic:
-        res[name] = dic[name]
-    return res
-
-
 def count(groupiter):
     return sum(1 for row in groupiter)
 
@@ -1304,6 +1276,16 @@ def detach_process():
     fork_then_exit_parent()
 
 
+def shortlist(lst):
+    """
+    >>> shortlist([1, 2, 3, 4, 5, 6, 7, 8])
+    '[1, 2, 3, ..., 6, 7, 8]'
+    """
+    if len(lst) <= 7:
+        return str(lst)
+    return str(lst[:3] + ['...'] + lst[-3:]).replace("'", "")
+
+
 def println(msg):
     """
     Convenience function to print messages on a single line in the terminal
@@ -1519,6 +1501,24 @@ def rmsdiff(a, b):
     rms = numpy.sqrt(((a - b)**2).mean(axis=axis))
     return rms
 
+
+def sqrscale(x_min, x_max, n):
+    """
+    :param x_min: minumum value
+    :param x_max: maximum value
+    :param n: number of steps
+    :returns: an array of n values from x_min to x_max in a quadratic scale
+    """
+    if not (isinstance(n, int) and n > 0):
+        raise ValueError('n must be a positive integer, got %s' % n)
+    if x_min < 0:
+        raise ValueError('x_min must be positive, got %s' % x_min)
+    if x_max <= x_min:
+        raise ValueError('x_max (%s) must be bigger than x_min (%s)' %
+                         (x_max, x_min))
+    delta = numpy.sqrt(x_max - x_min) / (n - 1)
+    return x_min + (delta * numpy.arange(n))**2
+
 # #################### COMPRESSION/DECOMPRESSION ##################### #
 
 # Compressing the task outputs makes everything slower, so you should NOT
