@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2022 GEM Foundation
+# Copyright (C) 2017-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -15,6 +15,20 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
+"""
+Here is the simplest client-server you can build with zeromq:
+
+# client
+with Socket('tcp://0.0.0.0:8000', zmq.REQ, 'connect') as sock:
+    for char in 'abc':
+        sock.send(char)
+
+# server
+with Socket('tcp://0.0.0.0:8000', zmq.REP, 'bind') as sock:
+    for msg in sock:
+        print(msg)
+        sock.send('ok')  # mandatory!
+"""
 import re
 import zmq
 import time
@@ -78,9 +92,11 @@ class Socket(object):
     :param end_point: zmq end point string
     :param socket_type: zmq socket type (integer)
     :param mode: default 'bind', accepts also 'connect'
-    :param timeout: default 15000 ms, used when polling the underlying socket
+    :param timeout: default 30s, used when polling the underlying socket
     """
-    def __init__(self, end_point, socket_type, mode, timeout=15000):
+    # NB: the timeout has to be large since starting a workerpool can be
+    # slow due to numba compiling everything, so you have to wait
+    def __init__(self, end_point, socket_type, mode, timeout=30):
         assert socket_type in (zmq.REP, zmq.REQ, zmq.PULL, zmq.PUSH)
         assert mode in ('bind', 'connect'), mode
         if mode == 'bind':
@@ -88,7 +104,7 @@ class Socket(object):
         self.end_point = end_point
         self.socket_type = socket_type
         self.mode = mode
-        self.timeout = timeout
+        self.timeout = timeout * 1000  # milliseconds
         self.running = False
 
     def __enter__(self):
@@ -159,11 +175,10 @@ class Socket(object):
             raise exc.__class__('%s: %r' % (exc, obj))
         self.num_sent += 1
         if self.socket_type == zmq.REQ:
-            ok = self.zsocket.poll(30000)  # 30 seconds
+            ok = self.zsocket.poll(self.timeout)
             if not ok:
-                raise TimeoutError(
-                    'While sending %s; probably the DbServer is off' %
-                    repr(obj))
+                raise TimeoutError('While sending %r to %s' %
+                                   (obj, self.end_point))
             return self.zsocket.recv_pyobj()
 
     def __repr__(self):
