@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2022 GEM Foundation
+# Copyright (C) 2014-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,21 +20,26 @@ import io
 import os
 import time
 import psutil
+import random
 import logging
 import operator
+import functools
 import numpy
 import pandas
 try:
     from PIL import Image
 except ImportError:
     Image = None
-from openquake.baselib import performance, parallel, hdf5, config
+from openquake.baselib import (
+    performance, parallel, hdf5, config, python3compat, workerpool as w)
 from openquake.baselib.general import (
     AccumDict, DictArray, block_splitter, groupby, humansize,
     get_nbytes_msg, agg_probs, pprod)
-from openquake.hazardlib.contexts import ContextMaker, read_cmakers, basename
+from openquake.hazardlib.contexts import (
+    ContextMaker, read_cmakers, basename, get_maxsize)
 from openquake.hazardlib.calc.hazard_curve import classical as hazclassical
-from openquake.hazardlib.probability_map import ProbabilityMap, poes_dt
+from openquake.hazardlib.probability_map import (
+    ProbabilityMap, poes_dt, combine_probs)
 from openquake.commonlib import calc
 from openquake.calculators import base, getters, extract
 
@@ -50,8 +55,64 @@ BUFFER = 1.5  # enlarge the pointsource_distance sphere to fix the weight;
 # with ps_grid_spacing=50
 get_weight = operator.attrgetter('weight')
 disagg_grp_dt = numpy.dtype([
-    ('grp_start', U16), ('grp_trt', hdf5.vstr), ('avg_poe', F32),
-    ('nsites', U32)])
+    ('grp_trt', hdf5.vstr), ('avg_poe', F32), ('nsites', U32)])
+slice_dt = numpy.dtype([('sid', U32), ('start', int), ('stop', int)])
+
+
+def get_pmaps_gb(dstore):
+    """
+    :returns: memory required on the master node to keep the pmaps
+    """
+    N = len(dstore['sitecol'])
+    L = dstore['oqparam'].imtls.size
+    cmakers = read_cmakers(dstore)
+    num_gs = [len(cm.gsims) for cm in cmakers]
+    return sum(num_gs) * N * L * 8 / 1024**3
+
+
+def build_slice_by_sid(sids, offset=0):
+    """
+    Convert an array of site IDs (with repetitions) into an array slice_dt
+    """
+    arr = performance.idx_start_stop(sids)
+    sbs = numpy.zeros(len(arr), slice_dt)
+    sbs['sid'] = arr[:, 0]
+    sbs['start'] = arr[:, 1] + offset
+    sbs['stop'] = arr[:, 2] + offset
+    return sbs
+
+
+def _concat(acc, slc2):
+    if len(acc) == 0:
+        return [slc2]
+    slc1 = acc[-1]  # last slice
+    if slc2[0] == slc1[1]:
+        new = numpy.array([slc1[0], slc2[1]])
+        return acc[:-1] + [new]
+    return acc + [slc2]
+
+
+def compactify(arrayN2):
+    """
+    :param arrayN2: an array with columns (start, stop)
+    :returns: a shorter array with the same structure
+
+    Here is how it works in an example where the first three slices
+    are compactified into one while the last slice stays as it is:
+
+    >>> arr = numpy.array([[84384702, 84385520],
+    ...                    [84385520, 84385770],
+    ...                    [84385770, 84386062],
+    ...                    [84387636, 84388028]])
+    >>> compactify(arr)
+    array([[84384702, 84386062],
+           [84387636, 84388028]])
+    """
+    if len(arrayN2) == 1:
+        # nothing to compactify
+        return arrayN2
+    out = numpy.array(functools.reduce(_concat, arrayN2, []))
+    return out
 
 
 class Set(set):
@@ -105,7 +166,7 @@ def semicolon_aggregate(probs, source_ids):
     srcids = [srcid.split(';')[0] for srcid in source_ids]
     unique, indices = numpy.unique(srcids, return_inverse=True)
     new = numpy.zeros_like(probs)
-    for i, s1, s2 in performance._idx_start_stop(indices):
+    for i, s1, s2 in performance.idx_start_stop(indices):
         new[..., i] = pprod(probs[..., s1:s2], axis=-1)
     return new, unique
 
@@ -117,39 +178,41 @@ def check_disagg_by_src(dstore):
     info = dstore['source_info'][:]
     mutex = info['mutex_weight'] > 0
     mean = dstore.sel('hcurves-stats', stat='mean')[:, 0]  # N, M, L
-    dbs = dstore.sel('disagg_by_src')  # N, R, M, L, Ns
+    dbs = dstore.sel('disagg_by_src')  # N, R, M, L1, Ns
     if mutex.sum():
         dbs_indep = dbs[:, :, :, :, ~mutex]
         dbs_mutex = dbs[:, :, :, :, mutex]
-        poes_indep = pprod(dbs_indep, axis=4)  # N, R, M, L
-        poes_mutex = dbs_mutex.sum(axis=4)  # N, R, M, L
+        poes_indep = pprod(dbs_indep, axis=4)  # N, R, M, L1
+        poes_mutex = dbs_mutex.sum(axis=4)  # N, R, M, L1
         poes = poes_indep + poes_mutex - poes_indep * poes_mutex
     else:
-        poes = pprod(dbs, axis=4)  # N, R, M, L
+        poes = pprod(dbs, axis=4)  # N, R, M, L1
     rlz_weights = dstore['weights'][:]
-    mean2 = numpy.einsum('sr...,r->s...', poes, rlz_weights)  # N, M, L
+    mean2 = numpy.einsum('sr...,r->s...', poes, rlz_weights)  # N, M, L1
     if not numpy.allclose(mean, mean2, atol=1E-6):
-        logging.error('check_disagg_src fails: %s =! %s',
-                      mean[0], mean2[0])
+        logging.error('check_disagg_src fails: %s =! %s', mean[0], mean2[0])
 
     # check the extract call is not broken
-    aw = extract.extract(dstore, 'disagg_by_src?lvl_id=-1')
-    assert aw.array.dtype.names == ('src_id', 'poe')
+    for imt in dstore['oqparam'].imtls:
+        aw = extract.extract(dstore, f'disagg_by_src?imt={imt}&poe=1E-4')
+        assert aw.array.dtype.names == ('src_id', 'poe')
 
 #  ########################### task functions ############################ #
 
 
-def classical(srcs, sids, cmaker, monitor):
+def classical(srcs, sitecol, cmaker, monitor):
     """
-    Read the sitecol and call the classical calculator in hazardlib
+    Call the classical calculator in hazardlib
     """
     cmaker.init_monitoring(monitor)
-    sitecol = monitor.read('sitecol')
-    if sids is not None:
-        sitecol = sitecol.filter(numpy.isin(sitecol.sids, sids))
-    result = hazclassical(srcs, sitecol, cmaker)
-    # print(srcs, sum(src.weight for src in srcs))
-    return result
+    rup_indep = getattr(srcs, 'rup_interdep', None) != 'mutex'
+    for sites in sitecol.split_in_tiles(cmaker.ntiles):
+        pmap = ProbabilityMap(
+            sites.sids, cmaker.imtls.size, len(cmaker.gsims)).fill(rup_indep)
+        result = hazclassical(srcs, sites, cmaker, pmap)
+        result['pnemap'] = ~pmap.remove_zeros()
+        result['pnemap'].gidx = cmaker.gidx
+        yield result
 
 
 def postclassical(pgetter, N, hstats, individual_rlzs,
@@ -167,6 +230,9 @@ def postclassical(pgetter, N, hstats, individual_rlzs,
     The "kind" is a string of the form 'rlz-XXX' or 'mean' of 'quantile-XXX'
     used to specify the kind of output.
     """
+    if 20 < monitor.task_no < pgetter.ntasks - 20:
+        # give time to the other tasks
+        time.sleep(pgetter.ntasks * random.random())
     with monitor('read PoEs', measuremem=True):
         pgetter.init()
 
@@ -177,26 +243,24 @@ def postclassical(pgetter, N, hstats, individual_rlzs,
                            for imt in pgetter.imtls})
     else:
         imtls = pgetter.imtls
-    poes, weights = pgetter.poes, pgetter.weights
+    poes, weights, sids = pgetter.poes, pgetter.weights, U32(pgetter.sids)
     M = len(imtls)
-    P = len(poes)
     L = imtls.size
+    L1 = L // M
     R = len(weights)
     S = len(hstats)
     pmap_by_kind = {}
     if R > 1 and individual_rlzs or not hstats:
-        pmap_by_kind['hcurves-rlzs'] = [ProbabilityMap(L) for r in range(R)]
-        if poes:
-            pmap_by_kind['hmaps-rlzs'] = [
-                ProbabilityMap(M, P) for r in range(R)]
+        pmap_by_kind['hcurves-rlzs'] = [
+            ProbabilityMap(sids, M, L1).fill(0) for r in range(R)]
     if hstats:
-        pmap_by_kind['hcurves-stats'] = [ProbabilityMap(L) for r in range(S)]
-        if poes:
-            pmap_by_kind['hmaps-stats'] = [
-                ProbabilityMap(M, P) for r in range(S)]
+        pmap_by_kind['hcurves-stats'] = [
+            ProbabilityMap(sids, M, L1).fill(0) for r in range(S)]
     combine_mon = monitor('combine pmaps', measuremem=False)
     compute_mon = monitor('compute stats', measuremem=False)
-    for sid in pgetter.sids:
+    sidx = ProbabilityMap(sids, 1, 1).fill(0).sidx
+    for sid in sids:
+        idx = sidx[sid]
         with combine_mon:
             pc = pgetter.get_pcurve(sid)  # shape (L, R)
             if amplifier:
@@ -205,20 +269,22 @@ def postclassical(pgetter, N, hstats, individual_rlzs,
         if pc.array.sum() == 0:  # no data
             continue
         with compute_mon:
+            if R > 1 and individual_rlzs or not hstats:
+                for r in range(R):
+                    pmap_by_kind['hcurves-rlzs'][r].array[idx] = (
+                        pc.array[:, r].reshape(M, L1))
             if hstats:
                 for s, (statname, stat) in enumerate(hstats.items()):
                     sc = getters.build_stat_curve(pc, imtls, stat, weights)
-                    pmap_by_kind['hcurves-stats'][s][sid] = sc
-                    if poes:
-                        hmap = calc.make_hmap(sc, imtls, poes, sid)
-                        pmap_by_kind['hmaps-stats'][s].update(hmap)
-            if R > 1 and individual_rlzs or not hstats:
-                for r, pmap in enumerate(pmap_by_kind['hcurves-rlzs']):
-                    pmap[sid] = pc.extract(r)
-                if poes:
-                    for r in range(R):
-                        hmap = calc.make_hmap(pc.extract(r), imtls, poes, sid)
-                        pmap_by_kind['hmaps-rlzs'][r].update(hmap)
+                    arr = sc.array.reshape(M, L1)
+                    pmap_by_kind['hcurves-stats'][s].array[idx] = arr
+
+    if poes and (R > 1 and individual_rlzs or not hstats):
+        pmap_by_kind['hmaps-rlzs'] = calc.make_hmaps(
+            pmap_by_kind['hcurves-rlzs'], imtls, poes)
+    if poes and hstats:
+        pmap_by_kind['hmaps-stats'] = calc.make_hmaps(
+            pmap_by_kind['hcurves-stats'], imtls, poes)
     return pmap_by_kind
 
 
@@ -250,45 +316,73 @@ class Hazard:
     """
     def __init__(self, dstore, full_lt, srcidx):
         self.datastore = dstore
+        oq = dstore['oqparam']
         self.full_lt = full_lt
+        self.weights = numpy.array(
+            [r.weight['weight'] for r in full_lt.get_realizations()])
         self.cmakers = read_cmakers(dstore, full_lt)
-        self.imtls = imtls = dstore['oqparam'].imtls
-        self.level_weights = imtls.array.flatten() / imtls.array.sum()
+        self.collect_rlzs = oq.collect_rlzs
+        self.totgsims = sum(len(cm.gsims) for cm in self.cmakers)
+        self.imtls = oq.imtls
+        self.level_weights = oq.imtls.array.flatten() / oq.imtls.array.sum()
         self.sids = dstore['sitecol/sids'][:]
         self.srcidx = srcidx
         self.N = len(dstore['sitecol/sids'])
+        self.L = oq.imtls.size
         self.R = full_lt.get_num_paths()
         self.acc = AccumDict(accum={})
+        self.offset = 0
 
-    def get_hcurves(self, pmap, rlzs_by_gsim):  # used in in disagg_by_src
+    def get_poes(self, pmap, cmaker):  # used in in disagg_by_src
         """
         :param pmap: a ProbabilityMap
-        :param rlzs_by_gsim: a dictionary gsim -> rlz IDs
-        :returns: an array of PoEs of shape (N, R, M, L)
+        :param cmaker: a ContextMaker
+        :returns: an array of PoEs of shape (N, R, M, L1)
         """
-        res = numpy.zeros((self.N, self.R, self.imtls.size))
-        for sid, pc in pmap.items():
-            for gsim_idx, rlzis in enumerate(rlzs_by_gsim.values()):
-                poes = pc.array[:, gsim_idx]
-                for rlz in rlzis:
-                    res[sid, rlz] = agg_probs(res[sid, rlz], poes)
-        return res.reshape(self.N, self.R, len(self.imtls), -1)
+        R = 1 if self.collect_rlzs else self.R
+        M = len(self.imtls)
+        L1 = self.L // M
+        out = numpy.zeros((self.N, R, M, L1))
+        dic = dict(zip(cmaker.gidx, cmaker.gsims.values()))
+        for lvl in range(self.L):
+            m, l = divmod(lvl, L1)
+            res = numpy.zeros((len(pmap.sids), self.R))
+            for i, g in enumerate(pmap.gidx):
+                combine_probs(res, pmap.array[:, lvl, i], U32(dic[g]))
+            if self.collect_rlzs:
+                out[:, 0, m, l] = res @ self.weights
+            else:
+                out[:, :, m, l] = res
+        return out
 
-    def store_poes(self, grp_id, pmap):
+    def store_poes(self, g, pnes, pnes_sids):
         """
-        Store the pmap of the given group inside the _poes dataset
+        Store 1-pnes inside the _poes dataset
         """
-        cmaker = self.cmakers[grp_id]
-        arr = pmap.array(self.N)
-        # arr[arr < 1E-5] = 0.  # minimum_poe
-        sids, lids, gids = arr.nonzero()
-        hdf5.extend(self.datastore['_poes/sid'], sids)
-        hdf5.extend(self.datastore['_poes/gid'], gids + cmaker.start)
-        hdf5.extend(self.datastore['_poes/lid'], lids)
-        hdf5.extend(self.datastore['_poes/poe'], arr[sids, lids, gids])
-        self.acc[grp_id]['grp_start'] = cmaker.start
-        self.acc[grp_id]['avg_poe'] = arr.mean(axis=(0, 2))@self.level_weights
-        self.acc[grp_id]['nsites'] = len(pmap)
+        # Physically, an extremely small intensity measure level can have an
+        # extremely large probability of exceedence, however that probability
+        # cannot be exactly 1 unless the level is exactly 0. Numerically, the
+        # PoE can be 1 and this give issues when calculating the damage (there
+        # is a log(0) in
+        # :class:`openquake.risklib.scientific.annual_frequency_of_exceedence`).
+        # Here we solve the issue by replacing the unphysical probabilities 1
+        # with .9999999999999999 (the float64 closest to 1).
+        poes = 1. - pnes
+        poes[poes == 1.] = .9999999999999999
+        # poes[poes < 1E-5] = 0.  # minimum_poe
+        idxs, lids = poes.nonzero()
+        gids = numpy.repeat(g, len(idxs))
+        if len(idxs):
+            sids = pnes_sids[idxs]
+            hdf5.extend(self.datastore['_poes/sid'], sids)
+            hdf5.extend(self.datastore['_poes/gid'], gids)
+            hdf5.extend(self.datastore['_poes/lid'], lids)
+            hdf5.extend(self.datastore['_poes/poe'], poes[idxs, lids])
+            sbs = build_slice_by_sid(sids, self.offset)
+            hdf5.extend(self.datastore['_poes/slice_by_sid'], sbs)
+            self.offset += len(sids)
+        self.acc[g]['avg_poe'] = poes.mean(axis=0) @ self.level_weights
+        self.acc[g]['nsites'] = len(pnes_sids)
 
     def store_disagg(self, pmaps=None):
         """
@@ -301,17 +395,15 @@ class Hazard:
             if dic:
                 trti, smrs = numpy.divmod(indices, n)
                 trt = self.full_lt.trts[trti[0]]
-                lst.append((dic['grp_start'], trt, dic['avg_poe'],
-                            dic['nsites']))
+                lst.append((trt, dic['avg_poe'], dic['nsites']))
         self.datastore['disagg_by_grp'] = numpy.array(lst, disagg_grp_dt)
         if pmaps:  # called inside a loop
             disagg_by_src = self.datastore['disagg_by_src'][()]
             for key, pmap in pmaps.items():
                 if isinstance(key, str):
                     # in case of disagg_by_src key is a source ID
-                    rlzs_by_gsim = self.cmakers[pmap.grp_id].gsims
                     disagg_by_src[..., self.srcidx[key]] = (
-                        self.get_hcurves(pmap, rlzs_by_gsim))
+                        self.get_poes(pmap, self.cmakers[pmap.grp_id]))
             self.datastore['disagg_by_src'][:] = disagg_by_src
 
 
@@ -322,7 +414,7 @@ class ClassicalCalculator(base.HazardCalculator):
     """
     core_task = classical
     precalc = 'preclassical'
-    accept_precalc = ['preclassical', 'classical']
+    accept_precalc = ['preclassical', 'classical', 'aftershock']
     SLOW_TASK_ERROR = False
 
     def agg_dicts(self, acc, dic):
@@ -341,34 +433,43 @@ class ClassicalCalculator(base.HazardCalculator):
         self.source_data += sdata
         grp_id = dic.pop('grp_id')
         self.rel_ruptures[grp_id] += sum(sdata['nrupts'])
-        self.cfactor += dic.pop('cfactor')
+        cfactor = dic.pop('cfactor')
+        if cfactor[1] != cfactor[0]:
+            print('ctxs_per_mag = {:.0f}, cfactor_per_task = {:.1f}'.format(
+                cfactor[1] / cfactor[2], cfactor[1] / cfactor[0]))
+        self.cfactor += cfactor
 
         # store rup_data if there are few sites
         if self.few_sites and len(dic['rup_data']):
             with self.monitor('saving rup_data'):
                 store_ctxs(self.datastore, dic['rup_data'], grp_id)
 
-        pmap = dic['pmap']
-        pmap.grp_id = grp_id
-        pmap_by_src = dic.pop('pmap_by_src', {})
+        pnemap = dic['pnemap']  # probabilities of no exceedence
+        pmap_by_src = dic.pop('pmap_by_src', {})  # non-empty for disagg_by_src
         # len(pmap_by_src) > 1 only for mutex sources, see contexts.py
         for source_id, pm in pmap_by_src.items():
             # store the poes for the given source
             acc[source_id] = pm
             pm.grp_id = grp_id
-        if pmap:
-            acc[grp_id] |= pmap
-        self.n_outs[grp_id] -= 1
-        if self.n_outs[grp_id] == 0:  # no other tasks for this grp_id
-            with self.monitor('storing PoEs', measuremem=True):
-                self.haz.store_poes(grp_id, acc.pop(grp_id))
+            pm.gidx = pnemap.gidx
+        for i, g in enumerate(pnemap.gidx):
+            if g in acc:
+                acc[g].update(pnemap, i)
+                self.n_outs[g] -= 1
+                assert self.n_outs[g] > -1, (g, self.n_outs[g])
+                if self.n_outs[g] == 0:  # no other tasks for this g
+                    with self.monitor('storing PoEs', measuremem=True):
+                        pne = acc.pop(g)
+                        self.haz.store_poes(g, pne.array[:, :, 0], pne.sids)
+            else:  # single output
+                with self.monitor('storing PoEs', measuremem=True):
+                    self.haz.store_poes(g, pnemap.array[:, :, i], pnemap.sids)
         return acc
 
     def create_dsets(self):
         """
         Store some empty datasets in the datastore
         """
-        self.init_poes()
         params = {'grp_id', 'occurrence_rate', 'clon', 'clat', 'rrup',
                   'probs_occur', 'sids', 'src_id', 'rup_id', 'weight'}
         gsims_by_trt = self.full_lt.get_gsims_by_trt()
@@ -393,8 +494,6 @@ class ClassicalCalculator(base.HazardCalculator):
                     dt = F32
                 descr.append((param, dt))
             self.datastore.create_df('rup', descr, 'gzip')
-        self.cfactor = numpy.zeros(2)
-        self.rel_ruptures = AccumDict(accum=0)  # grp_id -> rel_ruptures
         # NB: the relevant ruptures are less than the effective ruptures,
         # which are a preclassical concept
         if self.oqparam.disagg_by_src:
@@ -408,20 +507,25 @@ class ClassicalCalculator(base.HazardCalculator):
         self.M = len(oq.imtls)
         self.L1 = oq.imtls.size // self.M
         sources = list(self.csm.source_info)
+        R = 1 if oq.collect_rlzs else self.R
         size, msg = get_nbytes_msg(
-            dict(N=self.N, R=self.R, M=self.M, L1=self.L1, Ns=len(sources)))
+            dict(N=self.N, R=R, M=self.M, L1=self.L1, Ns=len(sources)))
         if size > TWO32:
-            raise RuntimeError('The matrix disagg_by_src is too large: %s'
-                               % msg)
+            raise RuntimeError('The matrix disagg_by_src is too large, '
+                               'use collect_rlzs=true\n%s' % msg)
+        size = self.N * R * self.M * self.L1 * len(sources) * 8
+        logging.info('Creating disagg_by_src of size %s', humansize(size))
         self.datastore.create_dset(
             'disagg_by_src', F32,
-            (self.N, self.R, self.M, self.L1, len(sources)))
+            (self.N, R, self.M, self.L1, len(sources)))
         self.datastore.set_shape_descr(
-            'disagg_by_src', site_id=self.N, rlz_id=self.R,
+            'disagg_by_src', site_id=self.N, rlz_id=R,
             imt=list(self.oqparam.imtls), lvl=self.L1, src_id=sources)
         return sources
 
     def init_poes(self):
+        self.cfactor = numpy.zeros(3)
+        self.rel_ruptures = AccumDict(accum=0)  # grp_id -> rel_ruptures
         if self.oqparam.hazard_calculation_id:
             full_lt = self.datastore.parent['full_lt']
             trt_smrs = self.datastore.parent['trt_smrs'][:]
@@ -435,20 +539,21 @@ class ClassicalCalculator(base.HazardCalculator):
             for rlzs in rlzs_by_gsim.values():
                 rlzs_by_g.append(rlzs)
         self.datastore.create_df('_poes', poes_dt.items())
+        self.datastore.create_dset('_poes/slice_by_sid', slice_dt)
         # NB: compressing the dataset causes a big slowdown in writing :-(
-        if not self.oqparam.hazard_calculation_id:
-            self.datastore.swmr_on()
 
-    def check_memory(self, N, L, num_gs):
+    def check_memory(self, N, L, maxw):
         """
         Log the memory required to receive the largest ProbabilityMap,
         assuming all sites are affected (upper limit)
         """
-        G = max(num_gs)
-        size = G * N * L * 8
-        # for ESHM20 there are 95,000 sites and up to 72,000 can be affected
-        logging.info('Requiring %s for full ProbabilityMap of shape %s',
-                     humansize(size), (G, N, L))
+        num_gs = [len(cm.gsims) for cm in self.cmakers]
+        max_gs = max(num_gs)
+        maxsize = get_maxsize(len(self.oqparam.imtls), max_gs)
+        logging.info('Considering {:_d} contexts at once'.format(maxsize))
+        size = max_gs * N * L * 8
+        logging.info('ProbabilityMap(G=%d,N=%d,L=%d) %s per core',
+                     max_gs, N, L, humansize(size))
         avail = min(psutil.virtual_memory().available, config.memory.limit)
         if avail < size:
             raise MemoryError(
@@ -464,52 +569,130 @@ class ClassicalCalculator(base.HazardCalculator):
         if oq.hazard_calculation_id:
             parent = self.datastore.parent
             if '_poes' in parent:
-                self.post_classical()  # repeat post-processing
+                self.build_curves_maps()  # repeat post-processing
                 return {}
             else:  # after preclassical, like in case_36
+                logging.info('Reading from parent calculation')
                 self.csm = parent['_csm']
+                oq.mags_by_trt = {
+                    trt: python3compat.decode(dset[:])
+                    for trt, dset in parent['source_mags'].items()}
                 self.full_lt = parent['full_lt']
                 self.datastore['source_info'] = parent['source_info'][:]
-                max_weight = self.csm.get_max_weight(oq)
+                maxw = self.csm.get_max_weight(oq)
         else:
-            max_weight = self.max_weight
-        self.create_dsets()  # create the rup/ datasets BEFORE swmr_on()
+            maxw = self.max_weight
+        self.init_poes()
         srcidx = {
             rec[0]: i for i, rec in enumerate(self.csm.source_info.values())}
         self.haz = Hazard(self.datastore, self.full_lt, srcidx)
-        # only groups generating more than 1 task preallocate memory
-        num_gs = [len(cm.gsims) for grp, cm in enumerate(self.haz.cmakers)]
-        L = oq.imtls.size
-        tiles = self.sitecol.split_max(oq.max_sites_per_tile)
-        if len(tiles) > 1:
-            sizes = [len(tile) for tile in tiles]
-            logging.info('There are %d tiles of sizes %s', len(tiles), sizes)
-            for size in sizes:
-                assert size > oq.max_sites_disagg, (size, oq.max_sites_disagg)
         self.source_data = AccumDict(accum=[])
-        self.n_outs = AccumDict(accum=0)
-        acc = {}
+        if not performance.numba:
+            logging.warning('numba is not installed: using the slow algorithm')
+
+        self.cmakers = self.haz.cmakers
+
         t0 = time.time()
-        for t, tile in enumerate(tiles, 1):
-            self.check_memory(len(tile), L, num_gs)
-            sids = tile.sids if len(tiles) > 1 else None
-            smap = self.submit(sids, self.haz.cmakers, max_weight)
-            for cm in self.haz.cmakers:
-                acc[cm.grp_id] = ProbabilityMap.build(L, len(cm.gsims))
-            smap.reduce(self.agg_dicts, acc)
-            if len(tiles) > 1:
-                logging.info('Finished tile %d of %d', t, len(tiles))
+        req = get_pmaps_gb(self.datastore)
+        ntiles = 1 + int(req / (oq.pmap_max_gb * 100))  # 50 GB
+        self.n_outs = AccumDict(accum=0)
+        if ntiles > 1:
+            self.execute_seq(maxw, ntiles)
+        else:  # regular case
+            self.check_memory(len(self.sitecol), oq.imtls.size, maxw)
+            self.execute_par(maxw)
         self.store_info()
-        self.haz.store_disagg(acc)
         if self.cfactor[0] == 0:
             raise RuntimeError('Filtered away all ruptures??')
         logging.info('cfactor = {:_d}/{:_d} = {:.1f}'.format(
             int(self.cfactor[1]), int(self.cfactor[0]),
             self.cfactor[1] / self.cfactor[0]))
+        if '_poes' in self.datastore:
+            self.build_curves_maps()
         if not oq.hazard_calculation_id:
             self.classical_time = time.time() - t0
         return True
 
+    def execute_par(self, maxw):
+        """
+        Regular case
+        """
+        self.create_dsets()  # create the rup/ datasets BEFORE swmr_on()
+        acc = self.run_one(self.sitecol, maxw)
+        self.haz.store_disagg(acc)
+
+    def execute_seq(self, maxw, ntiles):
+        """
+        Use sequential tiling
+        """
+        assert self.N > self.oqparam.max_sites_disagg, self.N
+        logging.info('Running %d tiles', ntiles)
+        for n, tile in enumerate(self.sitecol.split(ntiles)):
+            if n == 0:
+                self.check_memory(len(tile), self.oqparam.imtls.size, maxw)
+            self.run_one(tile, maxw)
+            logging.info('Finished tile %d of %d', n+1, ntiles)
+            if parallel.oq_distribute() == 'zmq':
+                w.WorkerMaster(config.zworkers).restart()
+            else:
+                parallel.Starmap.shutdown()
+
+    def run_one(self, sitecol, maxw):
+        """
+        Run a subset of sites and update the accumulator
+        """
+        acc = {}  # g -> pmap
+        oq = self.oqparam
+        L = oq.imtls.size
+        allargs = []
+        for cm in self.cmakers:
+            G = len(cm.gsims)
+            sg = self.csm.src_groups[cm.grp_id]
+
+            # maximum size of the pmap array in GB
+            size_gb = G * L * len(sitecol) * 8 / 1024**3
+            ntiles = int(numpy.ceil(size_gb / oq.pmap_max_gb))
+            # NB: disagg_by_src is disabled in case of tiling
+            assert not (ntiles > 1 and oq.disagg_by_src)
+            # NB: tiling only works with many sites
+            if ntiles > 1 and self.N < oq.max_sites_disagg * ntiles:
+                raise RuntimeError('There are not enough sites (%d) for '
+                                   '%d tiles' % (self.N, ntiles))
+            cm.ntiles = ntiles
+            if ntiles > 1:
+                logging.debug('Producing %d inner tiles', ntiles)
+
+            if sg.atomic or sg.weight <= maxw:
+                for g in cm.gidx:
+                    self.n_outs[g] += cm.ntiles
+                allargs.append((sg, sitecol, cm))
+            else:
+                if oq.disagg_by_src:  # possible only with a single tile
+                    blks = groupby(sg, basename).values()
+                else:
+                    blks = block_splitter(sg, maxw, get_weight, sort=True)
+                for block in blks:
+                    logging.debug('Sending %d source(s) with weight %d',
+                                  len(block), sg.weight)
+                    for g in cm.gidx:
+                        self.n_outs[g] += cm.ntiles
+                    allargs.append((block, sitecol, cm))
+
+            # allocate memory
+            for g in cm.gidx:
+                if self.n_outs[g] > 1:
+                    acc[g] = ProbabilityMap(sitecol.sids, L, 1).fill(1)
+
+        totsize = sum(pmap.array.nbytes for pmap in acc.values())
+        logging.info('Global pmap size %s', humansize(totsize))
+
+        self.datastore.swmr_on()  # must come before the Starmap
+        smap = parallel.Starmap(classical, h5=self.datastore.hdf5)
+        # using submit avoids the .task_queue and thus core starvation
+        for args in allargs:
+            smap.submit(args)
+        return smap.reduce(self.agg_dicts, acc)
+
     def store_info(self):
         """
         Store full_lt, source_info and source_data
@@ -527,39 +710,6 @@ class ClassicalCalculator(base.HazardCalculator):
         self.datastore.create_df('source_data', df)
         self.source_data.clear()  # save a bit of memory
 
-    def submit(self, sids, cmakers, max_weight):
-        """
-        :returns: a Starmap instance for the current tile
-        """
-        oq = self.oqparam
-        self.datastore.swmr_on()  # must come before the Starmap
-        smap = parallel.Starmap(classical, h5=self.datastore.hdf5)
-        smap.monitor.save('sitecol', self.sitecol)
-        triples = []
-        for grp_id in self.grp_ids:
-            sg = self.csm.src_groups[grp_id]
-            if sg.atomic:
-                # do not split atomic groups
-                trip = (sg, sids, cmakers[grp_id])
-                triples.append(trip)
-                smap.submit(trip)
-                self.n_outs[grp_id] += 1
-            else:  # regroup the sources in blocks
-                blks = (groupby(sg, basename).values()
-                        if oq.disagg_by_src else
-                        block_splitter(
-                            sg, max_weight, get_weight, sort=True))
-                blocks = list(blks)
-                for block in blocks:
-                    logging.debug(
-                        'Sending %d source(s) with weight %d',
-                        len(block), sum(src.weight for src in block))
-                    trip = (block, sids, cmakers[grp_id])
-                    triples.append(trip)
-                    smap.submit(trip)
-                    self.n_outs[grp_id] += 1
-        return smap
-
     def collect_hazard(self, acc, pmap_by_kind):
         """
         Populate hcurves and hmaps in the .hazard dictionary
@@ -576,18 +726,14 @@ class ClassicalCalculator(base.HazardCalculator):
                 array = self.hazard[kind]
             else:
                 dset = self.datastore.getitem(kind)
-                array = self.hazard[kind] = numpy.zeros(
-                    dset.shape, dset.dtype)
+                array = self.hazard[kind] = numpy.zeros(dset.shape, dset.dtype)
             for r, pmap in enumerate(pmaps):
-                for s in pmap:
-                    if kind.startswith('hmaps'):
-                        array[s, r] = pmap[s].array  # shape (M, P)
-                    else:  # hcurves
-                        array[s, r] = pmap[s].array.reshape(-1, self.L1)
+                for idx, sid in enumerate(pmap.sids):
+                    array[sid, r] = pmap.array[idx]  # shape (M, P)
 
     def post_execute(self, dummy):
         """
-        Compute the statistical hazard curves
+        Check for slow tasks and fix disagg_by_src if needed
         """
         task_info = self.datastore.read_df('task_info', 'taskname')
         try:
@@ -602,19 +748,19 @@ class ClassicalCalculator(base.HazardCalculator):
                 raise RuntimeError('%s in #%d' % (msg, self.datastore.calc_id))
             elif slow_tasks:
                 logging.info(msg)
-        if self.oqparam.disagg_by_src and '_poes' in self.datastore:
-            srcids = list(self.csm.source_info)
+        if 'disagg_by_src' in list(self.datastore):
+            srcids = python3compat.decode(
+                self.datastore['source_info']['source_id'])
             if any(';' in srcid for srcid in srcids):
                 # enable reduction of the array disagg_by_src
-                arr = self.datastore['disagg_by_src'][:]
+                arr = self.disagg_by_src = self.datastore['disagg_by_src'][:]
                 arr, srcids = semicolon_aggregate(arr, srcids)
                 self.datastore['disagg_by_src'][:] = arr
+                R = 1 if self.oqparam.collect_rlzs else self.R
                 self.datastore.set_shape_descr(
-                    'disagg_by_src', site_id=self.N, rlz_id=self.R,
+                    'disagg_by_src', site_id=self.N, rlz_id=R,
                     imt=list(self.oqparam.imtls), lvl=self.L1, src_id=srcids)
-        if '_poes' in self.datastore:
-            self.post_classical()
-        if 'disagg_by_src' in self.datastore:
+        if 'disagg_by_src' in self.datastore and not self.oqparam.collect_rlzs:
             logging.info('Comparing disagg_by_src vs mean curves')
             check_disagg_by_src(self.datastore)
 
@@ -658,26 +804,33 @@ class ClassicalCalculator(base.HazardCalculator):
                     imt=list(oq.imtls), poe=oq.poes)
         return N, S, M, P, L1, individual_rlzs
 
-    def post_classical(self):
+    # called by execute before post_execute
+    def build_curves_maps(self):
         """
-        Store hcurves-rlzs, hcurves-stats, hmaps-rlzs, hmaps-stats
+        Compute and store hcurves-rlzs, hcurves-stats, hmaps-rlzs, hmaps-stats
         """
         oq = self.oqparam
         hstats = oq.hazard_stats()
         if not oq.hazard_curves:  # do nothing
             return
         N, S, M, P, L1, individual = self._create_hcurves_maps()
-        ct = oq.concurrent_tasks or 1
-        if 1 < ct < 80:  # saving memory on small machines
-            ct = 80
+        poes_gb = self.datastore.getsize('_poes') / 1024**3
+        if poes_gb < .1:
+            ct = 1
+        elif poes_gb < 2:
+            ct = int(poes_gb * 10)
+        else:
+            ct = int(poes_gb) + 18  # number of tasks > number of GB
+        if ct > 1:
+            logging.info('Producing %d postclassical tasks', ct)
         self.weights = ws = [rlz.weight for rlz in self.realizations]
         if '_poes' in set(self.datastore):
             dstore = self.datastore
         else:
             dstore = self.datastore.parent
         sites_per_task = int(numpy.ceil(self.N / ct))
-        nbytes = len(dstore['_poes/sid']) * 4
-        logging.info('Reading %s of _poes/sid', humansize(nbytes))
+        sbs = dstore['_poes/slice_by_sid'][:]
+        sbs['sid'] //= sites_per_task
         # NB: there is a genious idea here, to split in tasks by using
         # the formula ``taskno = sites_ids // sites_per_task`` and then
         # extracting a dictionary of slices for each taskno. This works
@@ -686,26 +839,34 @@ class ClassicalCalculator(base.HazardCalculator):
         # disposed in 2 groups and we want to produce 2 tasks we can use
         # 012345012345 // 3 = 000111000111 and the slices are
         # {0: [(0, 3), (6, 9)], 1: [(3, 6), (9, 12)]}
-        slicedic = performance.get_slices(
-            dstore['_poes/sid'][:] // sites_per_task)
+        slicedic = AccumDict(accum=[])
+        for idx, start, stop in sbs:
+            slicedic[idx].append((start, stop))
         if not slicedic:
             # no hazard, nothing to do, happens in case_60
             return
-        nslices = sum(len(slices) for slices in slicedic.values())
+
+        # using compactify improves the performance of `read PoEs`;
+        # I have measured a 3.5x in the AUS model with 1 rlz
+        allslices = [compactify(slices) for slices in slicedic.values()]
+        nslices = sum(len(slices) for slices in allslices)
         logging.info('There are %d slices of poes [%.1f per task]',
                      nslices, nslices / len(slicedic))
         allargs = [
-            (getters.PmapGetter(dstore, ws, slices, oq.imtls, oq.poes),
+            (getters.PmapGetter(dstore, ws, slices, oq.imtls, oq.poes, ct),
              N, hstats, individual, oq.max_sites_disagg, self.amplifier)
-            for slices in slicedic.values()]
+            for slices in allslices]
         self.hazard = {}  # kind -> array
         hcbytes = 8 * N * S * M * L1
         hmbytes = 8 * N * S * M * P if oq.poes else 0
         logging.info('Producing %s of hazard curves and %s of hazard maps',
                      humansize(hcbytes), humansize(hmbytes))
         if not performance.numba:
-            logging.info('numba is not installed: using the slow algorithm')
-        self.datastore.swmr_on()  # essential before Starmap
+            logging.warning('numba is not installed: using the slow algorithm')
+        if 'delta_rates' in self.datastore.parent:
+            pass  # do nothing for the aftershock calculator, avoids an error
+        else:  # in all the other cases
+            self.datastore.swmr_on()
         parallel.Starmap(
             postclassical, allargs,
             distribute='no' if self.few_sites else None,
