@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2022 GEM Foundation
+# Copyright (C) 2015-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,12 +20,11 @@
 Disaggregation calculator core functionality
 """
 import logging
-import operator
 import numpy
 
 from openquake.baselib import parallel, hdf5, performance
 from openquake.baselib.general import (
-    AccumDict, get_nbytes_msg, humansize, pprod, agg_probs, block_splitter)
+    AccumDict, get_nbytes_msg, humansize, pprod, agg_probs, gen_slices)
 from openquake.baselib.python3compat import encode
 from openquake.hazardlib import stats
 from openquake.hazardlib.calc import disagg
@@ -116,7 +115,7 @@ def compute_disagg(dstore, slc, cmaker, hmap4, magidx, bin_edges, monitor):
     :param magidx:
         magnitude bin indices
     :param bin_egdes:
-        a quartet (dist_edges, lon_edges, lat_edges, eps_edges)
+        a sextet (mag dist lon lat eps trt) edges
     :param monitor:
         monitor of the currently running job
     :returns:
@@ -137,7 +136,7 @@ def compute_disagg(dstore, slc, cmaker, hmap4, magidx, bin_edges, monitor):
         for (s, z), r in numpy.ndenumerate(hmap4.rlzs):
             if r in rlzs:
                 g_by_z[s][z] = g
-    eps3 = disagg._eps3(cmaker.truncation_level, cmaker.num_epsilon_bins)
+    eps3 = disagg._eps3(cmaker.truncation_level, bin_edges[4])  # eps edges
     imts = [from_string(im) for im in cmaker.imtls]
     for magi in numpy.unique(magidx):
         for ctxt in ctxs:
@@ -155,8 +154,8 @@ def compute_disagg(dstore, slc, cmaker, hmap4, magidx, bin_edges, monitor):
                 iml2 = dict(zip(imts, iml3))
                 with dis_mon:
                     # 7D-matrix #disbins, #lonbins, #latbins, #epsbins, M, P, Z
-                    matrix = weighted_disagg(close, cmaker, g_by_z[s],
-                                             iml2, eps3, s, bins, epsstar)
+                    matrix = disaggregate(close, cmaker, g_by_z[s],
+                                          iml2, eps3, s, bins, epsstar)
                     for m in range(M):
                         mat6 = matrix[..., m, :, :]
                         if mat6.any():
@@ -167,7 +166,7 @@ def compute_disagg(dstore, slc, cmaker, hmap4, magidx, bin_edges, monitor):
     # the matrices is fast and the data are not queuing up
 
 
-def weighted_disagg(close, cmaker, g_by_z, iml2, eps3, s, bins, epsstar):
+def disaggregate(close, cmaker, g_by_z, iml2, eps3, s, bins, epsstar):
     """
     :returns: a 7D disaggregation matrix, weighted if src_mutex is True
     """
@@ -177,13 +176,13 @@ def weighted_disagg(close, cmaker, g_by_z, iml2, eps3, s, bins, epsstar):
         # built as ctx['weight'] = src.mutex_weight in contexts.py
         ctxs = performance.split_array(close, close.src_id)
         weights = [ctx.weight[0] for ctx in ctxs]
-        mats = [disagg.disaggregate(ctx, cmaker, g_by_z,
-                                    iml2, eps3, s, bins, epsstar=epsstar)
+        mats = [disagg.disaggregate(ctx, cmaker, g_by_z, iml2, eps3, s, bins,
+                                    epsstar=epsstar)
                 for ctx in ctxs]
         return numpy.average(mats, weights=weights, axis=0)
     else:
-        return disagg.disaggregate(close, cmaker, g_by_z,
-                                   iml2, eps3, s, bins, epsstar=epsstar)
+        return disagg.disaggregate(close, cmaker, g_by_z, iml2, eps3, s, bins,
+                                   epsstar=epsstar)
 
 
 def get_outputs_size(shapedic, disagg_outputs):
@@ -228,7 +227,7 @@ class DisaggregationCalculator(base.HazardCalculator):
                 'The number of sites is to disaggregate is %d, but you have '
                 'max_sites_disagg=%d' % (self.N, few))
         all_edges, shapedic = disagg.get_edges_shapedic(
-            self.oqparam, self.sitecol, self.datastore['source_mags'])
+            self.oqparam, self.sitecol, self.datastore['source_mags'], self.R)
         *b, trts = all_edges
         T = len(trts)
         shape = [len(bin) - 1 for bin in
@@ -243,7 +242,6 @@ class DisaggregationCalculator(base.HazardCalculator):
 
     def execute(self):
         """Performs the disaggregation"""
-        self.pre_checks()
         return self.full_disaggregation()
 
     def get_curve(self, sid, rlzs):
@@ -276,7 +274,7 @@ class DisaggregationCalculator(base.HazardCalculator):
         if oq.rlz_index is None and oq.num_rlzs_disagg == 0:
             oq.num_rlzs_disagg = len(ws)  # 0 means all rlzs
         edges, self.shapedic = disagg.get_edges_shapedic(
-            oq, self.sitecol, self.datastore['source_mags'])
+            oq, self.sitecol, self.datastore['source_mags'], self.R)
         self.save_bin_edges(edges)
         self.full_lt = self.datastore['full_lt']
         self.poes_disagg = oq.poes_disagg or (None,)
@@ -344,12 +342,6 @@ class DisaggregationCalculator(base.HazardCalculator):
         rdata['magi'] = magi
         rdata['idx'] = numpy.arange(totrups)
         rdata['grp_id'] = dstore['rup/grp_id'][:]
-        trt_smrs = dstore['trt_smrs'][:]
-        rlzs_by_gsim = self.full_lt.get_rlzs_by_gsim_list(trt_smrs)
-        G = max(len(rbg) for rbg in rlzs_by_gsim)
-        maxw = 2 * 1024**3 / (16 * G * self.M)  # at max 2 GB
-        maxweight = min(
-            numpy.ceil(totrups / (oq.concurrent_tasks or 1)), maxw)
         task_inputs = []
         U = 0
         self.datastore.swmr_on()
@@ -361,15 +353,16 @@ class DisaggregationCalculator(base.HazardCalculator):
         # that would break the ordering of the indices causing an incredibly
         # worse performance, but visible only in extra-large calculations!
         cmakers = read_cmakers(self.datastore)
-        grp_ids = U32(rdata['grp_id'])
+        grp_ids = rdata['grp_id']
+        G = max(len(cmaker.gsims) for cmaker in cmakers)
         for grp_id, slices in performance.get_slices(grp_ids).items():
             cmaker = cmakers[grp_id]
             for start, stop in slices:
-                slc = slice(start, stop)
-                U = max(U, stop - start)
-                smap.submit((dstore, slc, cmaker, self.hmap4,
-                             magi[slc], self.bin_edges))
-                task_inputs.append((grp_id, stop - start))
+                for slc in gen_slices(start, stop, 50_000):
+                    U = max(U, slc.stop - slc.start)
+                    smap.submit((dstore, slc, cmaker, self.hmap4,
+                                 magi[slc], self.bin_edges))
+                    task_inputs.append((grp_id, stop - start))
 
         nbytes, msg = get_nbytes_msg(dict(M=self.M, G=G, U=U, F=2))
         logging.info('Maximum mean_std per task:\n%s', msg)
@@ -437,7 +430,7 @@ class DisaggregationCalculator(base.HazardCalculator):
         *self.bin_edges, self.trts = all_edges
         b = self.bin_edges
 
-        def a(bin_no):
+        def ll_edges(bin_no):
             # lon/lat edges for the sites, bin_no can be 2 or 3
             num_edges = len(b[bin_no][0])
             arr = numpy.zeros((self.N, num_edges))
@@ -446,8 +439,8 @@ class DisaggregationCalculator(base.HazardCalculator):
             return arr
         self.datastore['disagg-bins/Mag'] = b[0]
         self.datastore['disagg-bins/Dist'] = b[1]
-        self.datastore['disagg-bins/Lon'] = a(2)
-        self.datastore['disagg-bins/Lat'] = a(3)
+        self.datastore['disagg-bins/Lon'] = ll_edges(2)
+        self.datastore['disagg-bins/Lat'] = ll_edges(3)
         self.datastore['disagg-bins/Eps'] = b[4]
         self.datastore['disagg-bins/TRT'] = encode(self.trts)
 
