@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2022 GEM Foundation
+# Copyright (C) 2015-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -26,18 +26,21 @@ import pandas
 from openquake.baselib import hdf5, parallel
 from openquake.baselib.general import AccumDict, copyobj, humansize
 from openquake.hazardlib.probability_map import ProbabilityMap
-from openquake.hazardlib.stats import geom_avg_std, compute_pmap_stats
+from openquake.hazardlib.stats import geom_avg_std, compute_stats
 from openquake.hazardlib.calc.stochastic import sample_ruptures
 from openquake.hazardlib.gsim.base import ContextMaker, FarAwayRupture
 from openquake.hazardlib.calc.filters import nofilter, getdefault, SourceFilter
 from openquake.hazardlib.calc.gmf import GmfComputer
+from openquake.hazardlib.shakemap.conditioned_gmfs import ConditionedGmfComputer
 from openquake.hazardlib import InvalidFile
 from openquake.hazardlib.calc.stochastic import get_rup_array, rupture_dt
+from openquake.hazardlib.site import SiteCollection
 from openquake.hazardlib.source.rupture import (
     RuptureProxy, EBRupture, get_ruptures)
 from openquake.commonlib import (
     calc, util, logs, readinput, logictree, datastore)
 from openquake.risklib.riskinput import str2rsi, rsi2str
+from openquake.commonlib.calc import get_mean_curve
 from openquake.calculators import base, views
 from openquake.calculators.getters import (
     get_rupture_getters, sig_eps_dt, time_dt)
@@ -54,19 +57,6 @@ TWO32 = numpy.float64(2 ** 32)
 
 # ######################## GMF calculator ############################ #
 
-def get_mean_curves(dstore, imt):
-    """
-    Extract the mean hazard curves from the datastore, as an array of shape
-    (N, L1)
-    """
-    if 'hcurves-stats' in dstore:  # shape (N, S, M, L1)
-        arr = dstore.sel('hcurves-stats', stat='mean', imt=imt)
-    else:  # there is only 1 realization
-        arr = dstore.sel('hcurves-rlzs', rlz_id=0, imt=imt)
-    return arr[:, 0, 0, :]
-
-# ########################################################################## #
-
 
 def count_ruptures(src):
     """
@@ -113,13 +103,47 @@ def event_based(proxies, full_lt, oqparam, dstore, monitor):
                     continue
                 proxy.geom = rupgeoms[proxy['geom_id']]
                 ebr = proxy.to_ebr(cmaker.trt)  # after the geometry is set
-                try:
-                    computer = GmfComputer(
-                        ebr, srcfilter.sitecol.filtered(sids), cmaker,
-                        oqparam.correl_model, oqparam.cross_correl,
-                        oqparam._amplifier, oqparam._sec_perils)
-                except FarAwayRupture:
-                    continue
+                if "station_data" in oqparam.inputs:
+                    station_sites = dstore.read_df('station_sites')
+                    station_data = dstore.read_df('station_data')
+                    station_sites = SiteCollection.from_points(
+                        lons=station_sites.lon.values,
+                        lats=station_sites.lat.values)
+                    station_sitemodel = station_sites.assoc(
+                        sitecol, assoc_dist=None)
+                    station_sitecol = SiteCollection.from_points(
+                        lons=station_sites.lon,
+                        lats=station_sites.lat,
+                        sitemodel=station_sitemodel)
+                    stnfilter = SourceFilter(
+                        station_sitecol, oqparam.maximum_distance(trt))
+                    stnids = stnfilter.close_sids(proxy, trt)
+                    if len(stnids) < len(station_sites):
+                        logging.warning('%d stations filtered away',
+                                        len(station_sites) - len(stnids))
+                    if len(stnids) == 0:  # all stations filtered away
+                        continue
+                    try:
+                        computer = ConditionedGmfComputer(
+                            ebr, srcfilter.sitecol.filtered(sids), 
+                            stnfilter.sitecol.filtered(stnids), 
+                            station_data.loc[stnids], oqparam.observed_imts,
+                            cmaker, oqparam.correl_model, oqparam.cross_correl,
+                            oqparam.ground_motion_correlation_params,
+                            oqparam.number_of_ground_motion_fields,
+                            oqparam._amplifier, oqparam._sec_perils)
+                    except FarAwayRupture:
+                        # skip this rupture
+                        continue
+                else:
+                    try:
+                        computer = GmfComputer(
+                            ebr, srcfilter.sitecol.filtered(sids), cmaker,
+                            oqparam.correl_model, oqparam.cross_correl,
+                            oqparam._amplifier, oqparam._sec_perils)
+                    except FarAwayRupture:
+                        # skip this rupture
+                        continue
             with cmon:
                 data = computer.compute_all(sig_eps)
             dt = time.time() - t0
@@ -195,13 +219,6 @@ class EventBasedCalculator(base.HazardCalculator):
             self.datastore.create_dset('ruptures', rupture_dt)
             self.datastore.create_dset('rupgeoms', hdf5.vfloat32)
 
-    def acc0(self):
-        """
-        Initial accumulator, a dictionary rlz -> ProbabilityMap
-        """
-        self.L = self.oqparam.imtls.size
-        return {r: ProbabilityMap(self.L) for r in range(self.R)}
-
     def build_events_from_sources(self):
         """
         Prefilter the composite source model and store the source_info
@@ -275,6 +292,8 @@ class EventBasedCalculator(base.HazardCalculator):
         :param acc: accumulator dictionary
         :param result: an AccumDict with events, ruptures, gmfs and hcurves
         """
+        if result is None:  # instead of a dict
+            raise MemoryError('You ran out of memory!')
         sav_mon = self.monitor('saving gmfs')
         agg_mon = self.monitor('aggregating hcurves')
         primary = self.oqparam.get_primary_imtls()
@@ -287,6 +306,10 @@ class EventBasedCalculator(base.HazardCalculator):
                 [task_no] = numpy.unique(times['task_no'])
                 rupids = list(times['rup_id'])
                 self.datastore['gmf_data/time_by_rup'][rupids] = times
+                if self.N >= calc.SLICE_BY_EVENT_NSITES:
+                    sbe = calc.build_slice_by_event(
+                        df.eid.to_numpy(), self.offset)
+                    hdf5.extend(self.datastore['gmf_data/slice_by_event'], sbe)
                 hdf5.extend(dset, df.sid.to_numpy())
                 hdf5.extend(self.datastore['gmf_data/eid'], df.eid.to_numpy())
                 for m in range(len(primary)):
@@ -298,14 +321,11 @@ class EventBasedCalculator(base.HazardCalculator):
                 sig_eps = result.pop('sig_eps')
                 hdf5.extend(self.datastore['gmf_data/sigma_epsilon'], sig_eps)
                 self.offset += len(df)
-        if self.offset >= TWO32:
-            raise RuntimeError(
-                'The gmf_data table has more than %d rows' % TWO32)
         imtls = self.oqparam.imtls
         with agg_mon:
             for key, poes in result.get('hcurves', {}).items():
                 r, sid, imt = str2rsi(key)
-                array = acc[r].setdefault(sid, 0).array[imtls(imt), 0]
+                array = acc[r].array[sid, imtls(imt), 0]
                 array[:] = 1. - (1. - array) * (1. - poes)
         self.datastore.flush()
         return acc
@@ -328,11 +348,11 @@ class EventBasedCalculator(base.HazardCalculator):
             oq.mags_by_trt = {trt: ['%.2f' % rup.mag]}
             self.cmaker = ContextMaker(trt, rlzs_by_gsim, oq)
             if self.N > oq.max_sites_disagg:  # many sites, split rupture
-                ebrs = [EBRupture(copyobj(rup, rup_id=rup.rup_id + i),
+                ebrs = [EBRupture(copyobj(rup, seed=rup.seed + i),
                                   'NA', 0, G, e0=i * G, scenario=True)
                         for i in range(ngmfs)]
             else:  # keep a single rupture with a big occupation number
-                ebrs = [EBRupture(rup, 'NA', 0, G * ngmfs, rup.rup_id,
+                ebrs = [EBRupture(rup, 'NA', 0, G * ngmfs, rup.seed,
                                   scenario=True)]
             srcfilter = SourceFilter(self.sitecol, oq.maximum_distance(trt))
             aw = get_rup_array(ebrs, srcfilter)
@@ -405,6 +425,8 @@ class EventBasedCalculator(base.HazardCalculator):
             dstore.create_dset('gmf_data/sigma_epsilon', sig_eps_dt(oq.imtls))
             dstore.create_dset('gmf_data/time_by_rup',
                                time_dt, (nrups,), fillvalue=None)
+            if self.N >= calc.SLICE_BY_EVENT_NSITES:
+                dstore.create_dset('gmf_data/slice_by_event', calc.slice_dt)
 
         # event_based in parallel
         nr = len(dstore['ruptures'])
@@ -412,6 +434,14 @@ class EventBasedCalculator(base.HazardCalculator):
         scenario = 'scenario' in oq.calculation_mode
         proxies = [RuptureProxy(rec, scenario)
                    for rec in dstore['ruptures'][:]]
+        if "station_data" in oq.inputs:
+            # this is meant to be used in conditioned scenario calculations with
+            # a single rupture; we are taking the first copy of the rupture
+            # (remember: _read_scenario_ruptures makes num_gmfs copies to 
+            # parallelize, but the conditioning process is computationally 
+            # expensive, so we want to avoid repeating it num_gmfs times)
+            # TODO: this is ugly and must be improved upon!
+            proxies = proxies[0:1]
         full_lt = self.datastore['full_lt']
         dstore.swmr_on()  # must come before the Starmap
         smap = parallel.Starmap.apply_split(
@@ -422,7 +452,13 @@ class EventBasedCalculator(base.HazardCalculator):
             concurrent_tasks=oq.concurrent_tasks or 1,
             duration=oq.time_per_task,
             outs_per_task=oq.outs_per_task)
-        acc = smap.reduce(self.agg_dicts, self.acc0())
+        if oq.hazard_curves_from_gmfs:
+            self.L = oq.imtls.size
+            acc0 = {r: ProbabilityMap(self.sitecol.sids, self.L, 1).fill(0)
+                    for r in range(self.R)}
+        else:
+            acc0 = {}
+        acc = smap.reduce(self.agg_dicts, acc0)
         if 'gmf_data' not in dstore:
             return acc
         if oq.ground_motion_fields:
@@ -467,9 +503,9 @@ class EventBasedCalculator(base.HazardCalculator):
         self.datastore['avg_gmf'] = avg_gmf
         return rel_events
 
-    def post_execute(self, result):
+    def post_execute(self, pmap_by_rlz):
         oq = self.oqparam
-        if (not result or not oq.ground_motion_fields and not
+        if (not pmap_by_rlz or not oq.ground_motion_fields and not
                 oq.hazard_curves_from_gmfs):
             return
         N = len(self.sitecol.complete)
@@ -486,19 +522,14 @@ class EventBasedCalculator(base.HazardCalculator):
             rlzs = self.datastore['full_lt'].get_realizations()
             # compute and save statistics; this is done in process and can
             # be very slow if there are thousands of realizations
-            weights = [rlz.weight for rlz in rlzs]
+            weights = [rlz.weight['weight'] for rlz in rlzs]
             # NB: in the future we may want to save to individual hazard
             # curves if oq.individual_rlzs is set; for the moment we
             # save the statistical curves only
             hstats = oq.hazard_stats()
             S = len(hstats)
-            pmaps = list(result.values())
             R = len(weights)
-            if len(pmaps) != R:
-                # this should never happen, unless I break the
-                # logic tree reduction mechanism during refactoring
-                raise AssertionError('Expected %d pmaps, got %d' %
-                                     (len(weights), len(pmaps)))
+            pmaps = [p.reshape(N, M, L1) for p in pmap_by_rlz.values()]
             if oq.individual_rlzs:
                 logging.info('Saving individual hazard curves')
                 self.datastore.create_dset('hcurves-rlzs', F32, (N, R, M, L1))
@@ -513,15 +544,11 @@ class EventBasedCalculator(base.HazardCalculator):
                     self.datastore.set_shape_descr(
                         'hmaps-rlzs', site_id=N, rlz_id=R,
                         imt=list(oq.imtls), poe=oq.poes)
-                for r, pmap in enumerate(pmaps):
-                    arr = numpy.zeros((N, M, L1), F32)
-                    for sid in pmap:
-                        arr[sid] = pmap[sid].array.reshape(M, L1)
-                    self.datastore['hcurves-rlzs'][:, r] = arr
+                for r in range(R):
+                    self.datastore['hcurves-rlzs'][:, r] = pmaps[r].array
                     if oq.poes:
-                        hmap = calc.make_hmap(pmap, oq.imtls, oq.poes)
-                        for sid in hmap:
-                            ds[sid, r] = hmap[sid].array
+                        [hmap] = calc.make_hmaps([pmaps[r]], oq.imtls, oq.poes)
+                        ds[:, r] = hmap.array
 
             if S:
                 logging.info('Computing statistical hazard curves')
@@ -538,16 +565,15 @@ class EventBasedCalculator(base.HazardCalculator):
                         'hmaps-stats', site_id=N, stat=list(hstats),
                         imt=list(oq.imtls), poes=oq.poes)
                 for s, stat in enumerate(hstats):
-                    pmap = compute_pmap_stats(
-                        pmaps, [hstats[stat]], weights, oq.imtls)
-                    arr = numpy.zeros((N, M, L1), F32)
-                    for sid in pmap:
-                        arr[sid] = pmap[sid].array.reshape(M, L1)
-                    self.datastore['hcurves-stats'][:, s] = arr
+                    smap = ProbabilityMap(self.sitecol.sids, L1, M)
+                    [smap.array] = compute_stats(
+                        numpy.array([p.array for p in pmaps]),
+                        [hstats[stat]], weights)
+                    self.datastore['hcurves-stats'][:, s] = smap.array
                     if oq.poes:
-                        hmap = calc.make_hmap(pmap, oq.imtls, oq.poes)
-                        for sid in hmap:
-                            ds[sid, s] = hmap[sid].array
+                        [hmap] = calc.make_hmaps([smap], oq.imtls, oq.poes)
+                        ds[:, s] = hmap.array
+
         if self.datastore.parent:
             self.datastore.parent.open('r')
         if oq.compare_with_classical:  # compute classical curves
@@ -563,9 +589,10 @@ class EventBasedCalculator(base.HazardCalculator):
                 # the computation
                 self.cl.run()
                 engine.expose_outputs(self.cl.datastore)
+                all = slice(None)
                 for imt in oq.imtls:
-                    cl_mean_curves = get_mean_curves(self.datastore, imt)
-                    eb_mean_curves = get_mean_curves(self.datastore, imt)
+                    cl_mean_curves = get_mean_curve(self.datastore, imt, all)
+                    eb_mean_curves = get_mean_curve(self.datastore, imt, all)
                     self.rdiff, index = util.max_rel_diff_index(
                         cl_mean_curves, eb_mean_curves)
                     logging.warning(
