@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2017-2022 GEM Foundation
+# Copyright (C) 2017-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -40,7 +40,7 @@ from openquake.hazardlib.gsim.base import ContextMaker, read_cmakers
 from openquake.hazardlib.calc import disagg, stochastic, filters
 from openquake.hazardlib.stats import calc_stats
 from openquake.hazardlib.source import rupture
-from openquake.risklib.scientific import LOSSTYPE
+from openquake.risklib.scientific import LOSSTYPE, LOSSID
 from openquake.risklib.asset import tagset
 from openquake.commonlib import calc, util, oqvalidation, datastore, logictree
 from openquake.calculators import getters
@@ -119,11 +119,10 @@ def parse(query_string, info={}):
     {'kind': ['rlz-3'], 'imt': ['PGA'], 'site_id': [0], 'k': [3], 'rlzs': True}
     """
     qdic = parse_qs(query_string)
-    loss_types = info.get('loss_types', [])
     for key, val in sorted(qdic.items()):
         # convert site_id to an int, loss_type to an int, etc
         if key == 'loss_type':
-            qdic[key] = [loss_types[k] for k in val]
+            qdic[key] = [LOSSID[k] for k in val]
             qdic['lt'] = val
         else:
             qdic[key] = [lit_eval(v) for v in val]
@@ -240,7 +239,8 @@ def extract_realizations(dstore, dummy):
         gsims = dstore.getitem('full_lt/gsim_lt')['uncertainty']
         if 'shakemap' in oq.inputs:
             gsims = ["[FromShakeMap]"]
-        arr['branch_path'] = ['"%s"' % repr(gsim)[1:-1].replace('"', '""')
+        # NOTE: repr(gsim) has a form like "b'[ChiouYoungs2008]'"
+        arr['branch_path'] = ['"%s"' % repr(gsim)[2:-1].replace('"', '""')
                               for gsim in gsims]  # quotes Excel-friendly
     else:
         arr['branch_path'] = encode(rlzs['branch_path'])
@@ -717,14 +717,16 @@ def extract_csq_curves(dstore, what):
                              consequences=cols))
 
 
+# NB: used by QGIS but not by the exporters
+# tested in test_case_1_ins
 @extract.add('agg_curves')
 def extract_agg_curves(dstore, what):
     """
     Aggregate loss curves from the ebrisk calculator:
 
-    /extract/agg_curves?kind=stats,absolute=1&loss_type=occupants&occupancy=RES
+    /extract/agg_curves?kind=stats&absolute=1&loss_type=occupants&occupancy=RES
 
-    Returns an array of shape (P, S, ...)
+    Returns an array of shape (#periods, #stats) or (#periods, #rlzs)
     """
     info = get_info(dstore)
     qdic = parse(what, info)
@@ -739,17 +741,29 @@ def extract_agg_curves(dstore, what):
         raise ValueError('Expected tagnames=%s, got %s' %
                          (info['tagnames'], tagnames))
     tagvalues = [tagdict[t][0] for t in tagnames]
-    agg_id = -1
     if tagnames:
         lst = decode(dstore['agg_keys'][:])
         agg_id = lst.index(','.join(tagvalues))
-    kinds = list(info['stats'])
-    name = 'agg_curves-stats/' + lts[0]
-    units = dstore.get_attr(name, 'units')
-    shape_descr = hdf5.get_shape_descr(dstore.get_attr(name, 'json'))
-    units = dstore.get_attr(name, 'units')
-    rps = shape_descr['return_period']
-    arr = dstore[name][agg_id, k].T  # shape P, R
+    else:
+        agg_id = 0  # total aggregation
+    if qdic['rlzs']:
+        [li] = qdic['loss_type']  # loss type index
+        units = dstore.get_attr('aggcurves', 'units').split()
+        df = dstore.read_df('aggcurves', sel=dict(agg_id=agg_id, loss_id=li))
+        rps = list(df.return_period.unique())
+        P = len(rps)
+        R = len(qdic['kind'])
+        arr = numpy.zeros((P, R))
+        for rlz in df.rlz_id.unique():
+            # NB: df may contains zeros but there are no missing periods
+            # by construction (see build_aggcurves)
+            arr[:, rlz] = df[df.rlz_id == rlz].loss
+    else:
+        name = 'agg_curves-stats/' + lts[0]
+        shape_descr = hdf5.get_shape_descr(dstore.get_attr(name, 'json'))
+        rps = list(shape_descr['return_period'])
+        units = dstore.get_attr(name, 'units').split()
+        arr = dstore[name][agg_id, k].T  # shape P, R
     if qdic['absolute'] == [1]:
         pass
     elif qdic['absolute'] == [0]:
@@ -758,9 +772,9 @@ def extract_agg_curves(dstore, what):
     else:
         raise ValueError('"absolute" must be 0 or 1 in %s' % what)
     attrs = dict(shape_descr=['return_period', 'kind'] + tagnames)
-    attrs['return_period'] = list(rps)
-    attrs['kind'] = kinds
-    attrs['units'] = list(units)  # used by the QGIS plugin
+    attrs['return_period'] = rps
+    attrs['kind'] = qdic['kind']
+    attrs['units'] = units  # used by the QGIS plugin
     for tagname, tagvalue in zip(tagnames, tagvalues):
         attrs[tagname] = [tagvalue]
     if tagnames:
@@ -810,7 +824,7 @@ def extract_agg_damages(dstore, what):
         oq = dstore['oqparam']
         lti = oq.lti[loss_type]
         D = len(oq.limit_states) + 1
-        damages = dstore['damages-rlzs'][:, :, lti, :D]
+        damages = dstore['damages-rlzs'][:, :, lti]
     else:
         raise KeyError('No damages found in %s' % dstore)
     return _filter_agg(dstore['assetcol'], damages, tags)
@@ -832,10 +846,9 @@ def extract_aggregate(dstore, what):
     ridx = qdic['k'][0]
     lis = qdic.get('loss_type', [])  # list of indices
     if lis:
-        li = lis[0]
-        lts = [lt for lt, i in loss_types.items() if i == li]
-        array = dstore['avg_losses%s/%s' % (suffix, lts[0])][:, ridx]
-        aw = ArrayWrapper(assetcol.aggregateby(tagnames, array), {}, lts)
+        lt = LOSSTYPE[lis[0]]
+        array = dstore['avg_losses%s/%s' % (suffix, lt)][:, ridx]
+        aw = ArrayWrapper(assetcol.aggregateby(tagnames, array), {}, [lt])
     else:
         array = avglosses(dstore, loss_types, suffix[1:])[:, ridx]
         aw = ArrayWrapper(assetcol.aggregateby(tagnames, array), {},
@@ -1136,8 +1149,9 @@ def extract_disagg(dstore, what):
     # adapted from the nrml_converters
     disag_tup = tuple(label.split('_'))
     axis = [bins[k] for k in disag_tup]
-    # compute axis mid points
-    axis = [(ax[: -1] + ax[1:]) / 2. if ax.dtype == float
+
+    # compute axis mid points, except for the TRT axis
+    axis = [(ax[: -1] + ax[1:]) / 2. if ax.dtype != object
             else ax for ax in axis]
     if len(axis) == 1:  # i.e. Mag or Dist
         values = numpy.array([axis[0]] + list(matrix.T))  # i.e. shape (2, 3)
@@ -1186,20 +1200,31 @@ def norm(qdict, params):
 @extract.add('disagg_by_src')
 def extract_disagg_by_src(dstore, what):
     """
-    Extract the disagg_by_src information Example:
-    http://127.0.0.1:8800/v1/calc/30/extract/disagg_by_src?site_id=0&imt_id=0&rlz_id=0&lvl_id=-1
+    Extract the disagg_by_src information.
+    Example: http://127.0.0.1:8800/v1/calc/30/extract/disagg_by_src?site_id=0&imt=PGA&poe=.001
     """
     qdict = parse(what)
-    dic = hdf5.get_shape_descr(dstore['disagg_by_src'].attrs['json'])
-    src_id = dic['src_id']
-    f = norm(qdict, 'site_id rlz_id lvl_id imt_id'.split())
-    poe = dstore['disagg_by_src'][
-        f['site_id'], f['rlz_id'], f['imt_id'], f['lvl_id']]
+    dset = dstore['disagg_by_src']
+    oq = dstore['oqparam']
+    attr = hdf5.get_shape_descr(dset.attrs['json'])
+    src_id = attr['src_id']
+    [imt] = qdict['imt']
+    [poe] = qdict['poe']
+    [site_id] = qdict.get('site_id', ['0'])
+    site_id = int(site_id)
+    mean = dstore.sel('hcurves-stats', imt=imt, stat='mean', site_id=site_id)[0, 0, 0]
+    lvl_id = calc.get_lvl(mean, oq.imtls[imt], float(poe))
+    imt_id = list(oq.imtls).index(imt)
+    poes = dset[site_id, :, imt_id, lvl_id]  # shape (R, Ns)
+    if oq.collect_rlzs:  # already averaged
+        poes = poes[0, :]
+    else:  # compute the average
+        poes = dstore['weights'][:] @ poes
     arr = numpy.zeros(len(src_id), [('src_id', '<S16'), ('poe', '<f8')])
     arr['src_id'] = src_id
-    arr['poe'] = poe[:len(src_id)]
+    arr['poe'] = poes[:len(src_id)]
     arr.sort(order='poe')
-    return ArrayWrapper(arr[::-1], dict(json=hdf5.dumps(f)))
+    return ArrayWrapper(arr[::-1], dict(site_id=site_id, imt=imt, poe=poe))
 
 
 @extract.add('disagg_layer')
@@ -1218,11 +1243,11 @@ def extract_disagg_layer(dstore, what):
         kinds = oq.disagg_outputs
     sitecol = dstore['sitecol']
     poes_disagg = oq.poes_disagg or (None,)
+    realizations = numpy.array(dstore['full_lt'].get_realizations())
     edges, shapedic = disagg.get_edges_shapedic(
-        oq, sitecol, dstore['source_mags'])
+        oq, sitecol, dstore['source_mags'], len(realizations))
     dt = _disagg_output_dt(shapedic, kinds, oq.imtls, poes_disagg)
     out = numpy.zeros(len(sitecol), dt)
-    realizations = numpy.array(dstore['full_lt'].get_realizations())
     hmap4 = dstore['hmap4'][:]
     best_rlzs = dstore['best_rlzs'][:]
     arr = {kind: dstore['disagg/' + kind][:] for kind in kinds}
@@ -1376,9 +1401,9 @@ def extract_eids_by_gsim(dstore, what):
 @extract.add('risk_stats')
 def extract_risk_stats(dstore, what):
     """
-    Extract the risk statistics from a DataFrame.
+    Compute the risk statistics from a DataFrame with individual realizations
     Example:
-    http://127.0.0.1:8800/v1/calc/30/extract/risk_stats/aggcurves
+    http://127.0.0.1:8800/v1/calc/30/extract/risk_stats/aggrisk
     """
     oq = dstore['oqparam']
     stats = oq.hazard_stats()
