@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2022 GEM Foundation
+# Copyright (C) 2015-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -22,6 +22,7 @@ Utilities to build a report writer generating a .rst report for a calculation
 """
 import os
 import sys
+import getpass
 import logging
 import numpy
 import pandas
@@ -86,11 +87,12 @@ class ReportWriter(object):
             line = '-' * len(title)
             self.text += '\n'.join(['\n\n' + title, line, text])
 
-    def make_report(self):
+    def make_report(self, show_inputs=True):
         """Build the report and return a restructed text string"""
         oq, ds = self.oq, self.dstore
-        for name in ('params', 'inputs'):
-            self.add(name)
+        if show_inputs:
+            for name in ('params', 'inputs'):
+                self.add(name)
         if 'full_lt' in ds:
             self.add('required_params_per_trt')
         if 'rup_data' in ds:
@@ -131,7 +133,7 @@ def build_report(job_ini, output_dir=None):
     :param output_dir:
         the directory where the report is written (default the input directory)
     """
-    with logs.init('job', job_ini) as log:
+    with logs.init('job', job_ini, user_name=getpass.getuser()) as log:
         oq = log.get_oqparam()
         if 'source_model_logic_tree' in oq.inputs:
             oq.calculation_mode = 'preclassical'
@@ -155,5 +157,5 @@ def build_report(job_ini, output_dir=None):
         rw.save(report)
     except IOError as exc:  # permission error
         sys.stderr.write(str(exc) + '\n')
-    readinput.exposure = None  # ugly hack
+    readinput.Global.reset()  # ugly hack
     return report
