@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2022 GEM Foundation
+# Copyright (C) 2015-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -30,11 +30,8 @@ from openquake import commands
 
 # check for Python version
 PY_VER = sys.version_info[:2]
-if PY_VER < (3, 6):
-    sys.exit('Python 3.6+ is required, you are using %s', sys.executable)
-elif PY_VER == (3, 6):
-    print('Python 3.6 (%s) is not supported; the engine may not work correctly'
-          % sys.executable)
+if PY_VER < (3, 8):
+    sys.exit('Python 3.8+ is required, you are using %s', sys.executable)
 
 
 # sanity check, all display name keys must be exportable
