@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2019-2022 GEM Foundation
+# Copyright (C) 2019-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -16,8 +16,7 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
-import os
-import getpass
+
 from openquake.calculators.post_risk import post_aggregate as main
 main.calc_id = 'ID of the risk calculation'
 main.aggregate_by = 'comma-separated list of tag names'
