@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2022 GEM Foundation
+# Copyright (C) 2018-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -21,7 +21,7 @@ from functools import partial
 import numpy
 from openquake.hazardlib import nrml
 from openquake.hazardlib.geo.geodetic import geodetic_distance
-from openquake.commonlib import readinput, calc, logs, datastore
+from openquake.commonlib import readinput, logs, datastore
 from openquake.calculators.base import calculators
 from openquake.calculators.extract import extract, WebExtractor
 
@@ -47,7 +47,6 @@ class OpenQuake(object):
         self.get_site_collection = readinput.get_site_collection
         self.get_composite_source_model = readinput.get_composite_source_model
         self.get_exposure = readinput.get_exposure
-        self.make_hmap = calc.make_hmap
         self.geodetic_distance = geodetic_distance
         # TODO: more utilities will be added when deemed useful
 
