@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2022 GEM Foundation
+# Copyright (C) 2015-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -26,7 +26,7 @@ import h5py
 
 from openquake.baselib import hdf5, performance, general
 from openquake.commonlib.logs import (
-    get_datadir, get_calc_ids, get_last_calc_id, CALC_REGEX, dbcmd)
+    get_datadir, get_calc_ids, get_last_calc_id, CALC_REGEX, dbcmd, init)
 
 
 def hdf5new(datadir=None):
@@ -146,6 +146,17 @@ def new(calc_id, oqparam, datadir=None, mode=None):
     return dstore
 
 
+def build_dstore_log(description='custom calculation', parent=()):
+    """
+    :returns: DataStore instance associated to the .calc_id
+    """
+    dic = dict(description=description, calculation_mode='custom')
+    log = init('calc', dic)
+    dstore = new(log.calc_id, log.get_oqparam(validate=False))
+    dstore.parent = parent
+    return dstore, log
+
+
 class DataStore(collections.abc.MutableMapping):
     """
     DataStore class to store the inputs/outputs of a calculation on the
@@ -153,12 +164,10 @@ class DataStore(collections.abc.MutableMapping):
 
     Here is a minimal example of usage:
 
-    >>> from openquake.commonlib import logs
-    >>> params = {'calculation_mode': 'scenario', 'sites': '0 0'}
-    >>> with logs.init("calc", params) as log:
-    ...     ds = new(log.calc_id, log.get_oqparam())
-    ...     ds['example'] = 42
-    ...     print(ds['example'][()])
+    >>> dstore, log = build_dstore_log()
+    >>> with dstore, log:
+    ...     dstore['example'] = 42
+    ...     print(dstore['example'][()])
     42
 
     When reading the items, the DataStore will return a generator. The
@@ -238,6 +247,8 @@ class DataStore(collections.abc.MutableMapping):
         """
         self.close()  # flush everything
         self.open('a')
+        if self.parent != ():
+            self.parent.open('r')
         try:
             self.hdf5.swmr_mode = True
         except (ValueError, RuntimeError):  # already set
