@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2022 GEM Foundation
+# Copyright (C) 2018-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -32,7 +32,11 @@ def zip_all(directory, ini):
         for f in files:
             if f.endswith(ini):
                 path = os.path.join(cwd, f)
-                oq = oqvalidation.OqParam(**readinput.get_params(path))
+                try:
+                    oq = oqvalidation.OqParam(**readinput.get_params(path))
+                except ValueError as exc:
+                    print('skipping %s: %s' % (f, exc))
+                    continue
                 zips.extend(readinput.get_input_files(oq))
     total = sum(os.path.getsize(z) for z in zips)
     if os.path.exists('jobs.zip'):
