@@ -52,6 +52,12 @@ source_info_dt = numpy.dtype([
     ('trti', numpy.uint8),             # 8
 ])
 
+def gzpik(obj):
+    """
+    gzip and pickle a python object
+    """
+    gz = gzip.compress(pickle.dumps(obj, pickle.HIGHEST_PROTOCOL))
+    return numpy.frombuffer(gz, numpy.uint8)
 
 
 def fragmentno(src):
@@ -166,8 +172,7 @@ def get_csm(oq, full_lt, h5=None):
         allargs.append((fname, converter))
     smdict = parallel.Starmap(read_source_model, allargs, distribute=dist,
                               h5=h5 if h5 else None).reduce()
-    if len(smdict) > 1:  # really parallel
-        parallel.Starmap.shutdown()  # save memory
+    parallel.Starmap.shutdown()  # save memory
     fix_geometry_sections(smdict, h5)
     logging.info('Applying uncertainties')
     groups = _build_groups(full_lt, smdict)
@@ -253,14 +258,15 @@ def _build_groups(full_lt, smdict):
             smlt_dir, smdict, rlz.value[0].split())
         bset_values = full_lt.source_model_lt.bset_values(rlz.lt_path)
         if bset_values and bset_values[0][0].uncertainty_type == 'extendModel':
-            (bset, value), *bset_values = bset_values
-            extra, extra_ids = _groups_ids(smlt_dir, smdict, value.split())
-            common = source_ids & extra_ids
-            if common:
-                raise InvalidFile(
-                    '%s contains source(s) %s already present in %s' %
-                    (value, common, rlz.value))
-            src_groups.extend(extra)
+            while len(bset_values):
+                (bset, value), *bset_values = bset_values
+                extra, extra_ids = _groups_ids(smlt_dir, smdict, value.split())
+                common = source_ids & extra_ids
+                if common:
+                    raise InvalidFile(
+                        '%s contains source(s) %s already present in %s' %
+                        (value, common, rlz.value))
+                src_groups.extend(extra)
         for src_group in src_groups:
             trt_smr = full_lt.get_trt_smr(src_group.trt, rlz.ordinal)
             sg = apply_uncertainties(bset_values, src_group)
@@ -387,7 +393,7 @@ class CompositeSourceModel:
             cmaker = ContextMaker(trts[trti], rlzs_by_gsim, oq)
             cmaker.tom = tom.PoissonTOM(oq.investigation_time)
             cmaker.trti = trti
-            cmaker.start = start
+            cmaker.gidx = numpy.arange(start, start + len(rlzs_by_gsim))
             cmaker.grp_id = grp_id
             start += len(rlzs_by_gsim)
             cmakers.append(cmaker)
@@ -533,14 +539,21 @@ class CompositeSourceModel:
         return max_weight
 
     def __toh5__(self):
-        data = gzip.compress(pickle.dumps(self, pickle.HIGHEST_PROTOCOL))
-        logging.info(f'Storing {general.humansize(len(data))} '
+        G = len(self.src_groups)
+        arr = numpy.zeros(G + 1, hdf5.vuint8)
+        for grp_id, grp in enumerate(self.src_groups):
+            arr[grp_id] = gzpik(grp)
+        arr[G] = gzpik(self.source_info)
+        size = sum(len(val) for val in arr)
+        logging.info(f'Storing {general.humansize(size)} '
                      'of CompositeSourceModel')
-        return numpy.void(data), {}
+        return arr, {}
 
-    def __fromh5__(self, blob, attrs):
-        other = pickle.loads(gzip.decompress(blob))
-        vars(self).update(vars(other))
+    # tested in case_36
+    def __fromh5__(self, arr, attrs):
+        objs = [pickle.loads(gzip.decompress(a.tobytes())) for a in arr]
+        self.src_groups = objs[:-1]
+        self.source_info = objs[-1]
 
     def __repr__(self):
         """
