@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2022 GEM Foundation
+# Copyright (C) 2014-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -211,7 +211,7 @@ def _get_regional_term(region, C, imt, vs30, rrup):
             [150, 250, 350, 450, 600, 850, 1150, 2000],
             [C['a36'], C['a37'], C['a38'], C['a39'], C['a40'], C['a41'],
              C['a42'], C['a42']],
-            kind='linear')
+            fill_value='extrapolate', kind='linear')
         return f3(vs30) + C['a29'] * rrup
     else:  # California
         return 0.
