@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2022 GEM Foundation
+# Copyright (C) 2014-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -22,17 +22,23 @@ Module exports :class:`KothaEtAl2020`,
                :class:`KothaEtAl2020Slope`,
                :class:`KothaEtAl2020ESHM20`,
                :class:`KothaEtAl2020ESHM20SlopeGeology`
+               :class:`KothaEtAl2020regional`
 """
+import os
 import numpy as np
 from scipy.constants import g
-
+from shapely.geometry import Point, shape
+from shapely.prepared import prep
 from openquake.baselib.general import CallableDict
-from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
+from openquake.hazardlib.geo.packager import fiona
+from openquake.hazardlib.gsim.base import GMPE, CoeffsTable, add_alias
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA, from_string
 from openquake.hazardlib.gsim.nga_east import (get_tau_at_quantile, ITPL,
                                                TAU_EXECUTION, TAU_SETUP)
 
+DATA_FOLDER = os.path.join(os.path.dirname(__file__), 'Kotha_2020')
+
 CONSTANTS = {"Mref": 4.5, "Rref": 30., "Mh": 5.7,
              "h_D10": 4.0, "h_10D20": 8.0, "h_D20": 12.0}
 
@@ -99,6 +105,7 @@ def get_distance_coefficients_1(kind, c3, c3_epsilon, C, imt, sctx):
     existing tau_c3 distribution
     """
     if c3:
+
         # Use the c3 that has been defined on input
         return c3
     else:
@@ -113,7 +120,6 @@ def get_distance_coefficients_2(kind, c3, c3_epsilon, C, imt, sctx):
     Returns the c3 term. If c3 was input directly into the GMPE then
     this over-rides the c3 regionalisation. Otherwise the c3 and tau_c3
     are determined according to the region to which each site is assigned.
-
     Note that no regionalisation is defined for PGV and hence the
     default values from Kotha et al. (2020) are taken unless defined
     otherwise in the input c3
@@ -139,14 +145,37 @@ def get_distance_coefficients_2(kind, c3, c3_epsilon, C, imt, sctx):
     return c3_ + c3_epsilon * tau_c3
 
 
-def get_distance_term(kind, c3, c3_epsilon, C, ctx, rjb, imt):
+def get_distance_coefficients_3(att, delta_c3_epsilon, C, imt, sctx):
+    """
+    Return site-specific coefficient 'C3'. The function retrieves the
+    value of delta_c3 and the standard error of delta_c3 from the 'att'
+    geojson file depending on the location of site. This delta_c3 is
+    added to the generic coefficient 'c3' from the GMPE. A delta_c3_epsilon
+    value of +/- 1.6 gives the 95% confidence interval for delta_c3.
+    """
+    s = [(Point(lon, lat)) for lon, lat in zip(sctx.lon, sctx.lat)]
+    delta_c3 = np.zeros((len(sctx.lat), 2), dtype=float)
+    for i, feature in enumerate(att):
+        prepared_polygon = prep(shape(feature['geometry']))
+        contained = list(filter(prepared_polygon.contains, s))
+        if contained:
+            l = np.concatenate([np.where((sctx['lon'] == p.x) &
+                               (sctx['lat'] == p.y))[0] for p in contained])
+            delta_c3[l, 0] = feature['properties'][str(imt)]
+            delta_c3[l, 1] = feature['properties'][str(imt)+'_se']
+
+    return C["c3"] + delta_c3[:, 0] + delta_c3_epsilon * delta_c3[:, 1]
+
+
+def get_distance_term(kind, c3, c3_epsilon, C, ctx, imt):
     """
     Returns the distance attenuation factor
     """
     h = _get_h(C, ctx.hypo_depth)
-    rval = np.sqrt(rjb ** 2. + h ** 2.)
+    rval = np.sqrt(ctx.rjb ** 2. + h ** 2.)
     rref_val = np.sqrt(CONSTANTS["Rref"] ** 2. + h ** 2.)
-    c3 = get_distance_coefficients(kind, c3, c3_epsilon, C, imt, ctx)
+    if kind != 'regional':
+        c3 = get_distance_coefficients(kind, c3, c3_epsilon, C, imt, ctx)
     f_r = (C["c1"] + C["c2"] * (ctx.mag - CONSTANTS["Mref"])) *\
         np.log(rval / rref_val) + (c3 * (rval - rref_val) / 100.)
     return f_r
@@ -162,6 +191,28 @@ def get_magnitude_scaling(C, mag):
                     C["e1"] + C["b3"] * d_m)
 
 
+def get_dl2l(tec, ctx, imt, delta_l2l_epsilon):
+    """
+    Returns rupture source specific delta_l2l values. The method
+    retrieves the delta_l2l and standard error of delta_l2l values.
+    if delta_l2l_epsilon is provided, standard error of delta_c3
+    will be included. A delta_l2l_epsilon value of +/- 1.6 gives
+    the 95% confidence interval for delta_l2l.
+    """
+    f = [(Point(lon, lat)) for lon, lat in zip(ctx.hypo_lon, ctx.hypo_lat)]
+    dl2l = np.zeros((len(ctx.hypo_lon), 2), dtype=float)
+    for i, feature in enumerate(tec):
+        prepared_polygon = prep(shape(feature['geometry']))
+        contained = list(filter(prepared_polygon.contains, f))
+        if contained:
+            l = np.concatenate([np.where((ctx['hypo_lon'] == p.x) &
+                               (ctx['hypo_lat'] == p.y))[0] for p in contained])
+            dl2l[l, 0] = feature['properties'][str(imt)]
+            dl2l[l, 1] = feature['properties'][str(imt)+'_se']
+
+    return dl2l[:, 0] + delta_l2l_epsilon * dl2l[:, 1]
+
+
 def get_sigma_mu_adjustment(C, imt, ctx):
     """
     Returns the sigma_mu adjusment factor, which is taken as the
@@ -194,7 +245,7 @@ def get_site_amplification(kind, extra, C, ctx, imt):
     """
     if kind == "base":  # no site amplification
         ampl = 0.
-    elif kind == "site":
+    elif kind in {"site", "regional"}:
         # Render with respect to 800 m/s reference Vs30
         sref = np.log(ctx.vs30 / 800.)
         ampl = (C["g0_vs30"] + C["g1_vs30"] * sref +
@@ -257,7 +308,7 @@ def get_stddevs(kind, ergodic, phi_s2s, C, ctx, imt):
             phi_s2s[ctx.vs30measured] += C["phi_s2s_obs"]
             phi_s2s[np.logical_not(ctx.vs30measured)] += C["phi_s2s_inf"]
             phi = np.sqrt(phi ** 2. + phi_s2s ** 2.)
-        elif kind == 'site':
+        elif kind in {"site", "regional"}:
             phi = np.sqrt(phi ** 2.0 + C["phi_s2s_vs30"] ** 2.)
         elif kind == 'slope':
             phi = np.sqrt(phi ** 2. + C["phi_s2s_slope"] ** 2.)
@@ -272,41 +323,32 @@ class KothaEtAl2020(GMPE):
     """
     Implements the first complete version of the newly derived GMPE
     for Shallow Crustal regions using the Engineering Strong Motion Flatfile.
-
     Kotha, S. R., Weatherill, G., Bindi, D., Cotton F. (2020) "A regionally-
     adaptable ground-motion model for shallow crustal earthquakes in Europe.
     Bulletin of Earthquake Engineering, 18:4091-4125
-
     The GMPE is desiged for regional adaptation within a logic-tree framework,
     and as such contains several parameters that can be calibrated on input:
-
     1) Source-region scaling, a simple scalar factor that defines how much
     to increase or decrease the "regional average" ground motion in the region.
     This value is taken as the maximum of the source-region variability term
     (tau_l2l) and the statistical uncertainty (sigma_mu). The latter defines
     the within-model uncertainty owing to the data set from which the model is
     derived and only exceeds the former at large magnitudes
-
     2) Residual attenuation scaling "c3", a factor that controls the residual
     attenuation part of the model to make the ground motion decay more or less
     rapidly with distance than the regional average.
-
     Both factors are period dependent.
-
     The two adaptable factors can be controlled either by direct specification
     at input (in the form of an imt-dependent dictionary) or by a number of
     standard deviations multiplying the existing variance terms. The two
     approaches are mutually exclusive, with the directly specified parameters
     always being used if defined on input.
-
     In the core form of the GMPE no site term is included. This is added in the
     subclasses.
-
     :param float sigma_mu_epsilon:
         Parameter to control the source-region scaling as a number of
         standard deviations by which to multiply the source-region to source-
         region variance, max(tau_l2l, sigma_mu)
-
     :param float c3_epsilon:
         Parameter to control the residual attenuation scaling as a number
         of standard deviations by which to multiply the attenuation-region
@@ -315,16 +357,13 @@ class KothaEtAl2020(GMPE):
         attenuation as an instance of :class:
         `openquake.hazardlib.gsim.base.CoeffsTable`. If absent, the value is
         taken from the normal coefficients table.
-
     :param bool ergodic:
         Use the ergodic standard deviation (True) or non-ergodic standard
         deviation (False)
-
     :param dict dl2l:
         If specifying the source-region scaling directly, defines the
         increase or decrease of the ground motion in the form of an imt-
         dependent dictionary of delta L2L factors
-
     :param dict c3:
         If specifying the residual attenuation scaling directly, defines the
         apparent anelastic attenuation term, c3, as an imt-dependent
@@ -413,10 +452,15 @@ class KothaEtAl2020(GMPE):
                 extra['GEOLOGICAL_UNITS'] = self.GEOLOGICAL_UNITS
             else:
                 phi_s2s = None
-
-            mean[m] = (get_magnitude_scaling(C, ctx.mag) +
-                       get_distance_term(self.kind, self.c3, self.c3_epsilon,
-                                         C, ctx, ctx.rjb, imt) +
+            if self.kind == 'regional':
+                c3 = get_distance_coefficients_3(self.att,
+                                                 self.delta_c3_epsilon,
+                                                 C, imt, ctx)
+            else:
+                c3 = self.c3
+            fp = get_distance_term(self.kind, c3, self.c3_epsilon,
+                                   C, ctx, imt)
+            mean[m] = (get_magnitude_scaling(C, ctx.mag) + fp +
                        get_site_amplification(self.kind, extra, C, ctx, imt))
             # GMPE originally in cm/s/s - convert to g
             if imt.string.startswith(('PGA', 'SA')):
@@ -426,6 +470,11 @@ class KothaEtAl2020(GMPE):
             if self.dl2l:
                 # The source-region parameter is specified explicity
                 mean[m] += self.dl2l[imt]["dl2l"]
+
+            elif self.kind == 'regional':
+                dl2l = get_dl2l(self.tec, ctx, imt, self.delta_l2l_epsilon)
+                mean[m] += dl2l
+
             elif self.sigma_mu_epsilon:
                 # epistemic uncertainty factor (sigma_mu) multiplied by
                 # the number of standard deviations
@@ -475,6 +524,38 @@ class KothaEtAl2020(GMPE):
     """)
 
 
+class KothaEtAl2020regional(KothaEtAl2020):
+    """
+    Adaptation of the Kotha et al. (2020) GMPE using
+    the source and site specific adjustments.
+    """
+    experimental = True
+
+    #: Required rupture parameters are magnitude, hypocentral location
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'hypo_lat', 'hypo_lon', 'hypo_depth'}
+
+    #: Required site parameter are vs30, lat and lon of the site
+    REQUIRES_SITES_PARAMETERS = {'vs30', 'lat', 'lon'}
+
+    kind = "regional"
+
+    def __init__(self, delta_l2l_epsilon=0.0, delta_c3_epsilon=0.0,
+                 ergodic=True, c3=None, dl2l=None, **kwargs):
+        """
+        Instantiate setting the dl2l and c3 terms.
+        """
+        super().__init__(delta_l2l_epsilon=delta_l2l_epsilon,
+                         delta_c3_epsilon=delta_c3_epsilon,
+                         ergodic=ergodic, **kwargs)
+        self.delta_l2l_epsilon = delta_l2l_epsilon
+        self.delta_c3_epsilon = delta_c3_epsilon
+        self.ergodic = ergodic
+        attenuation_file = os.path.join(DATA_FOLDER, 'kotha_attenuation_regions.geojson')
+        self.att = np.array(fiona.open(attenuation_file), dtype=object)
+        tectonic_file = os.path.join(DATA_FOLDER, 'kotha_tectonic_regions.geojson')
+        self.tec = np.array(fiona.open(tectonic_file), dtype=object)
+
+
 class KothaEtAl2020Site(KothaEtAl2020):
     """
     Preliminary adaptation of the Kotha et al. (2020) GMPE using
@@ -610,14 +691,11 @@ class KothaEtAl2020ESHM20(KothaEtAl2020):
     Adaptation of the Kotha et al. (2020) GMPE for application to the
     2020 European Seismic Hazard Model, as described in Weatherill et al.
     (2020)
-
     Weatherill, G., Kotha, S. R. and Cotton, F. (2020) "A regionally-adaptable
     'scaled-backbone' ground motion logic tree for shallow seismicity in
     Europe: application to the 2020 European seismic hazard model". Bulletin
     of Earthquake Engineering, 18:5087 - 5117
-
     There are three key adaptations of the original Kotha et al. (2020) GMM:
-
     1) The use of the residual attenuation regions, which represent the five
     main sub-regions of Europe with similar attenuation characteristics. The
     assignment to a particular group is now a site-dependent property,
@@ -626,12 +704,10 @@ class KothaEtAl2020ESHM20(KothaEtAl2020):
     belongs (1 - 5) or else the default values (0). For each region an expected
     c3 and variance, tau_c3, are defined from which the resulting c3 is taken
     as a multiple of the number of standard deviations of tau_c3.
-
     2) The site amplification is defined using a two-segment piecewise linear
     linear function. This form of the GMPE defines the site in terms of a
     measured or inferred Vs30, with the total aleatory variability adjusted
     accordingly.
-
     3) A magnitude-dependent heteroskedastic aleatroy uncertainty model is
     used for the region-corrected between-event residuals and the site-
     corrected within event residuals. The former taken from the "global" tau
@@ -639,7 +715,6 @@ class KothaEtAl2020ESHM20(KothaEtAl2020):
     model of Al Atik (2015) adapted to the distribution of site-corrected
     within-event residuals determined by the original regression of Kotha et
     al. (2020).
-
     Al Atik, L. (2015) NGA-East: Ground-Motion Standard Deviation Models for
     Central and Eastern North America, PEER Technical Report, No 2015/07
     """
@@ -823,5 +898,81 @@ class KothaEtAl2020ESHM20SlopeGeology(KothaEtAl2020ESHM20):
     5.0000     0.01675139    0.01898142           0.02123988    0.02293046   -0.00517242    -0.04022363   -0.04159239   0.00326963
     6.0000     0.00884584    0.01248789           0.01338540    0.01433546   -0.00328602    -0.02576612   -0.02615088   0.00321300
     7.0000     0.00754320    0.01081496           0.00952506    0.01292858   -0.00247815    -0.02210686   -0.02284293   0.00365737
-    8.0000     0.01794388    0.01906271           0.01478150    0.02515701   -0.00401437    -0.03950798   -0.04270189   0.00480742 
+    8.0000     0.01794388    0.01906271           0.01478150    0.02515701   -0.00401437    -0.03950798   -0.04270189   0.00480742
     """)
+
+
+# Add aliases for the ESHM20 selection - shallow crustal sources
+eshm20_crust_lines = '''\
+ESHM20ShallowCrustVLowStressFastAtten -2.85697 -1.732051
+ESHM20ShallowCrustVLowStressMidAtten -2.85697 0.000000
+ESHM20ShallowCrustVLowStressSlowAtten -2.85697 1.732051
+ESHM20ShallowCrustLowStressFastAtten -1.35563 -1.732051
+ESHM20ShallowCrustLowStressMidAtten -1.35563 0.000000
+ESHM20ShallowCrustLowStressSlowAtten -1.35563 1.732051
+ESHM20ShallowCrustMidStressFastAtten 0.00000 -1.732051
+ESHM20ShallowCrustMidStressMidAtten 0.00000 0.000000
+ESHM20ShallowCrustMidStressSlowAtten 0.00000 1.732051
+ESHM20ShallowCrustHighStressFastAtten 1.35563 -1.732051
+ESHM20ShallowCrustHighStressMidAtten 1.35563 0.000000
+ESHM20ShallowCrustHighStressSlowAtten 1.35563 1.732051
+ESHM20ShallowCrustVHighStressFastAtten 2.85697 -1.732051
+ESHM20ShallowCrustVHighStressMidAtten 2.85697 0.000000
+ESHM20ShallowCrustVHighStressSlowAtten 2.85697 1.732051'''.splitlines()
+
+for line_shallow in eshm20_crust_lines:
+    alias, sig_mu_eps, c3_eps = line_shallow.split()
+    add_alias(alias, KothaEtAl2020ESHM20,
+              sigma_mu_epsilon=float(sig_mu_eps),
+              c3_epsilon=float(c3_eps))
+
+
+# Add aliases for the ESHM20 Iceland ground motion model
+# dl2l values
+ICELAND_dL2L = {
+    "IMTs": ['PGA', 'SA(0.01)', 'SA(0.025)', 'SA(0.04)', 'SA(0.05)',
+             'SA(0.07)', 'SA(0.1)', 'SA(0.15)', 'SA(0.2)', 'SA(0.25)',
+             'SA(0.3)', 'SA(0.35)', 'SA(0.4)', 'SA(0.45)', 'SA(0.5)',
+             'SA(0.6)', 'SA(0.7)', 'SA(0.75)', 'SA(0.8)', 'SA(0.9)', 'SA(1.0)',
+             'SA(1.2)', 'SA(1.4)', 'SA(1.6)', 'SA(1.8)', 'SA(2.0)', 'SA(2.5)',
+             'SA(3.0)', 'SA(3.5)', 'SA(4.0)', 'SA(4.5)', 'SA(5.0)', 'SA(6.0)',
+             'SA(7.0)', 'SA(8.0)'],
+    "VLow": [-1.320714,  -1.334702, -1.411144, -1.593190, -1.732470, -1.846624,
+             -1.869080, -1.666701, -1.370120, -1.239216, -1.107578, -0.953322,
+             -0.919893, -0.888578, -0.819694, -0.763689, -0.703255, -0.641188,
+             -0.592629, -0.560762, -0.453129, -0.320960, -0.235436, -0.159245,
+             -0.011255, 0.114595, 0.268825, 0.214030, 0.061128, -0.066012,
+             -0.016226, 0.005109, 0.144121, 0.125620, -0.081479],
+    "Low": [-0.787319, -0.800154, -0.858052, -0.973254, -1.065389, -1.139585,
+            -1.150332, -1.045077, -0.886725, -0.798828, -0.706965, -0.605590,
+            -0.558332, -0.514375, -0.453700, -0.394184, -0.332799, -0.273109,
+            -0.232198, -0.188936, -0.095642, 0.019582, 0.112757, 0.176902,
+            0.296832, 0.410135, 0.555383, 0.545697, 0.417368, 0.285042,
+            0.279085, 0.298731, 0.430396, 0.438246, 0.243113],
+    "Mid": [-0.305692, -0.317486, -0.358640, -0.413486, -0.463050, -0.501166,
+            -0.501340, -0.483784, -0.450245, -0.401182, -0.345233, -0.291607,
+            -0.231861, -0.176490, -0.123226, -0.060540, 0.001704, 0.059246,
+            0.093252, 0.146803, 0.227150, 0.327073, 0.427158, 0.480424,
+            0.575018, 0.676991, 0.814129, 0.845175, 0.739034, 0.602026,
+            0.545734, 0.563855, 0.688888, 0.720531, 0.536202],
+    "High": [0.175935, 0.165182, 0.140772, 0.146282, 0.139289, 0.137253,
+             0.147652, 0.077509, -0.013765, -0.003536, 0.016499, 0.022376,
+             0.094610, 0.161395, 0.207248, 0.273104, 0.336207, 0.391601,
+             0.418702, 0.482542, 0.549942, 0.634564, 0.741559, 0.783946,
+             0.853204, 0.943847, 1.072875, 1.144653, 1.060700, 0.919010,
+             0.812383, 0.828979, 0.947380, 1.002816, 0.829291],
+    "VHigh": [0.709330, 0.699730, 0.693864, 0.766218, 0.806370, 0.844292,
+              0.866400, 0.699133, 0.469630, 0.436852, 0.417112, 0.370108,
+              0.456171, 0.535598, 0.573242, 0.642609, 0.706663, 0.759680,
+              0.779133, 0.854368, 0.907429, 0.975106, 1.089752, 1.120093,
+              1.161291, 1.239387, 1.359433, 1.476320, 1.416940, 1.270064,
+              1.107694, 1.122601, 1.233655, 1.315442, 1.153883],
+}
+
+# Build the set of aliases
+for stress in list(ICELAND_dL2L)[1:]:
+    dl2l = dict(list(zip(ICELAND_dL2L["IMTs"], ICELAND_dL2L[stress])))
+    for c3_key, c3_eps in zip(["Fast", "Mid", "Slow"],
+                              [-1.732051, 0.0, 1.732051]):
+        alias = "ESHM20Iceland{:s}Stress{:s}Atten".format(stress, c3_key)
+        add_alias(alias, KothaEtAl2020ESHM20, dl2l=dl2l, c3_epsilon=c3_eps)
