@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2022 GEM Foundation
+# Copyright (C) 2018-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -82,15 +82,14 @@ def PolygonPatch(polygon, **kwargs):
             raise ValueError(
                 "A polygon or multi-polygon representation is required")
 
-    vertices = concatenate([
-        concatenate([asarray(t.exterior)[:, :2]] +
-                    [asarray(r)[:, :2] for r in t.interiors])
-        for t in polygon])
-    codes = concatenate([
-        concatenate([coding(t.exterior)] +
-                    [coding(r) for r in t.interiors]) for t in polygon])
-
-    return PathPatch(Path(vertices, codes),  **kwargs)
+    vertices, codes = [], []
+    for t in polygon:
+        vertices.append(concatenate([asarray(t.exterior.coords)[:, :2]] +
+                            [asarray(r.coords)[:, :2] for r in t.interiors]))
+        codes.append(concatenate([coding(t.exterior)] +
+                                 [coding(r) for r in t.interiors]))
+    return PathPatch(Path(concatenate(vertices), concatenate(codes)),
+                     **kwargs)
 
 
 def debug_plot(*polygons):
