@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2013-2022 GEM Foundation
+# Copyright (C) 2013-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -308,8 +308,11 @@ class RiskModel(object):
         lratios = self.loss_ratios[loss_type]
         imls = self.hazard_imtls[vf.imt]
         values = assets['value-' + loss_type].to_numpy()
+        rtime = self.risk_investigation_time or self.investigation_time
         lrcurves = numpy.array(
-            [scientific.classical(vf, imls, hazard_curve, lratios)] * n)
+            [scientific.classical(
+                vf, imls, hazard_curve, lratios,
+                self.investigation_time, rtime)] * n)
         return rescale(lrcurves, values)
 
     def classical_bcr(self, loss_type, assets, hazard,
@@ -328,12 +331,17 @@ class RiskModel(object):
         self.assets = assets
         vf = self.risk_functions[loss_type]
         imls = self.hazard_imtls[vf.imt]
+        rtime = self.risk_investigation_time or self.investigation_time
         curves_orig = functools.partial(
             scientific.classical, vf, imls,
-            loss_ratios=self.loss_ratios_orig[loss_type])
+            loss_ratios=self.loss_ratios_orig[loss_type],
+            investigation_time=self.investigation_time,
+            risk_investigation_time=rtime)
         curves_retro = functools.partial(
             scientific.classical, vf.retro, imls,
-            loss_ratios=self.loss_ratios_retro[loss_type])
+            loss_ratios=self.loss_ratios_retro[loss_type],
+            investigation_time=self.investigation_time,
+            risk_investigation_time=rtime)
         original_loss_curves = numpy.array([curves_orig(hazard)] * n)
         retrofitted_loss_curves = numpy.array([curves_retro(hazard)] * n)
 
@@ -529,6 +537,35 @@ class CompositeRiskModel(collections.abc.Mapping):
         self.consdict = consdict or {}  # new style consequences, by anything
         self.init()
 
+    def set_tmap(self, tmap):
+        """
+        Set the attribute .tmap if the risk IDs in the
+        taxonomy mapping are consistent with the fragility functions.
+        """
+        self.tmap = tmap
+        if 'consequence' not in self.oqparam.inputs:
+            return
+        csq_files = []
+        for fnames in self.oqparam.inputs['consequence'].values():
+            if isinstance(fnames, list):
+                csq_files.extend(fnames)
+            else:
+                csq_files.append(fnames)
+        cfs = '\n'.join(csq_files)
+        for loss_type in tmap:
+            for byname, coeffs in self.consdict.items():
+                # ex. byname = "losses_by_taxonomy"
+                if len(coeffs):
+                    consequence, tagname = byname.split('_by_')
+                    # the taxonomy map is a dictionary loss_type ->
+                    # [[(risk_taxon, weight]),...] for each asset taxonomy
+                    for pairs in tmap[loss_type][1:]:  # strip [('?', 1)]
+                        for risk_t, weight in pairs:
+                            try:
+                                coeffs[risk_t][loss_type]
+                            except KeyError as err:
+                                raise InvalidFile(
+                                    'Missing %s in\n%s' % (err, cfs))
     def check_risk_ids(self, inputs):
         """
         Check that there are no missing risk IDs for some risk functions
@@ -778,6 +815,7 @@ class CompositeRiskModel(collections.abc.Mapping):
         :returns: a dictionary keyed by extended loss type
         """
         rc = scientific.RiskComputer(self, asset_df)
+        # rc.pprint()
         # dic = rc.todict()
         # rc2 = get_riskcomputer(dic)
         # dic2 = rc2.todict()
@@ -809,6 +847,7 @@ class CompositeRiskModel(collections.abc.Mapping):
                                           if self.damage_states else [])
         attrs = dict(covs=self.covs, loss_types=loss_types,
                      limit_states=limit_states,
+                     consequences=self.get_consequences(),
                      tmap=repr(getattr(self, 'tmap', [])))
         rf = next(iter(self.values()))
         if hasattr(rf, 'loss_ratios'):
