@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2022 GEM Foundation
+# Copyright (C) 2012-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -23,6 +23,7 @@ import copy
 import bisect
 import itertools
 import collections
+from pprint import pprint
 from functools import lru_cache
 
 import numpy
@@ -50,7 +51,21 @@ structural+nonstructural_ins structural+contents_ins nonstructural+contents_ins
 structural+nonstructural+contents_ins
 structural_ins nonstructural_ins reinsurance'''.split())
 TOTLOSSES = [lt for lt in LOSSTYPE if '+' in lt]
-LTI = {lt: i for i, lt in enumerate(LOSSTYPE)}
+LOSSID = {lt: i for i, lt in enumerate(LOSSTYPE)}
+
+
+def _reduce(nested_dic):
+    # reduce lists of floats into empty lists inside a nested dictionary
+    # used for pretty printing purposes
+    out = {}
+    for k, dic in nested_dic.items():
+        if isinstance(dic, list) and not isinstance(dic[0], (str, bytes)):
+            out[k] = []
+        elif isinstance(dic, dict):
+            out[k] = _reduce(dic)
+        else:
+            out[k] = dic
+    return out
 
 
 def pairwise(iterable):
@@ -180,7 +195,6 @@ class VulnerabilityFunction(object):
             self.covs = numpy.array(covs)
         else:
             self.covs = numpy.zeros(self.imls.shape)
-
         for lr, cov in zip(self.mean_loss_ratios, self.covs):
             if lr == 0 and cov > 0:
                 msg = ("It is not valid to define a mean loss ratio = 0 "
@@ -238,14 +252,14 @@ class VulnerabilityFunction(object):
         Compute the survival probability based on the underlying
         distribution.
         """
+        # scipy does not handle correctly the limit case stddev = 0.
+        # In that case, when `mean` > 0 the survival function
+        # approaches to a step function, otherwise (`mean` == 0) we
+        # returns 0
+        if stddev == 0:
+            return numpy.piecewise(
+                loss_ratio, [loss_ratio > mean or not mean], [0, 1])
         if self.distribution_name == 'LN':
-            # scipy does not handle correctly the limit case stddev = 0.
-            # In that case, when `mean` > 0 the survival function
-            # approaches to a step function, otherwise (`mean` == 0) we
-            # returns 0
-            if stddev == 0:
-                return numpy.piecewise(
-                    loss_ratio, [loss_ratio > mean or not mean], [0, 1])
             variance = stddev ** 2.0
             sigma = numpy.sqrt(numpy.log((variance / mean ** 2.0) + 1.0))
             mu = mean ** 2.0 / numpy.sqrt(variance + mean ** 2.0)
@@ -916,14 +930,24 @@ def scenario_damage(fragility_functions, gmvs):
 
 def annual_frequency_of_exceedence(poe, t_haz):
     """
-    :param poe: array of probabilities of exceedence
+    :param poe: array of probabilities of exceedence in time t_haz
     :param t_haz: hazard investigation time
     :returns: array of frequencies (with +inf values where poe=1)
     """
-    assert not (poe == 1).any()  # avoid log(0)
+    # replace 1 with the closest to 1 float64 number to avoid log(1-1)
+    poe[poe == 1.] = .9999999999999999
     return - numpy.log(1-poe) / t_haz
 
 
+def probability_of_exceedance(afoe, t_risk):
+    """
+    :param afoe: array of annual frequencies of exceedence
+    :param t_risk: risk investigation time
+    :returns: array of probabilities of exceedance in time t_risk
+    """
+    return 1 - numpy.exp(-t_risk * afoe)
+
+
 def classical_damage(
         fragility_functions, hazard_imls, hazard_poes,
         investigation_time, risk_investigation_time,
@@ -955,23 +979,27 @@ def classical_damage(
     else:
         imls = hazard_imls
         poes = numpy.array(hazard_poes)
-    afe = annual_frequency_of_exceedence(poes, investigation_time)
-    annual_frequency_of_occurrence = pairwise_diff(
-        pairwise_mean([afe[0]] + list(afe) + [afe[-1]]))
+
+    # convert the hazard probabilities of exceedance to
+    # annual frequencies of exceedance, and then occurrence
+    afoes = annual_frequency_of_exceedence(poes, investigation_time)
+    afoos = pairwise_diff(
+        pairwise_mean([afoes[0]] + list(afoes) + [afoes[-1]]))
     poes_per_damage_state = []
     for ff in fragility_functions:
-        fx = annual_frequency_of_occurrence @ ff(imls)
+        fx = afoos @ ff(imls)
         poe_per_damage_state = 1. - numpy.exp(-fx * risk_investigation_time)
         poes_per_damage_state.append(poe_per_damage_state)
     poos = pairwise_diff([1] + poes_per_damage_state + [0])
     return poos
 
 #
-# Classical
+# Classical Risk
 #
 
 
-def classical(vulnerability_function, hazard_imls, hazard_poes, loss_ratios):
+def classical(vulnerability_function, hazard_imls, hazard_poes, loss_ratios,
+              investigation_time, risk_investigation_time):
     """
     :param vulnerability_function:
         an instance of
@@ -983,6 +1011,10 @@ def classical(vulnerability_function, hazard_imls, hazard_poes, loss_ratios):
         the hazard curve
     :param loss_ratios:
         a tuple of C loss ratios
+    :param investigation_time:
+        hazard investigation time
+    :param risk_investigation_time:
+        risk investigation time
     :returns:
         an array of shape (2, C)
     """
@@ -999,13 +1031,23 @@ def classical(vulnerability_function, hazard_imls, hazard_poes, loss_ratios):
 
     # interpolate the hazard curve
     poes = interpolate.interp1d(hazard_imls, hazard_poes)(imls)
+    if abs((1-poes).mean()) < 1E-4:  # flat curve
+        raise ValueError('The hazard curve is flat (all ones) probably due to '
+                         'a (hazard) investigation time too large')
 
-    # compute the poos
-    pos = pairwise_diff(poes)
-    lrem_po = numpy.empty(lrem.shape)
-    for idx, po in enumerate(pos):
-        lrem_po[:, idx] = lrem[:, idx] * po  # column * po
-    return numpy.array([loss_ratios, lrem_po.sum(axis=1)])
+    # convert the hazard probabilities of exceedance ot annual
+    # frequencies of exceedance, and then occurrence
+    afoes = annual_frequency_of_exceedence(poes, investigation_time)
+    afoos = pairwise_diff(afoes)
+
+    # compute the annual frequency of exceedance of the loss ratios
+    # lrem = loss ratio exceedance matrix
+    lr_afoes = numpy.empty(lrem.shape)
+    for idx, afoo in enumerate(afoos):
+        lr_afoes[:, idx] = lrem[:, idx] * afoo  # column * afoo
+    lr_poes = probability_of_exceedance(
+        lr_afoes.sum(axis=1), risk_investigation_time)
+    return numpy.array([loss_ratios, lr_poes])
 
 
 # used in classical_risk only
@@ -1064,11 +1106,11 @@ def conditional_loss_ratio(loss_ratios, poes, probability):
 # Insured Losses
 #
 
-def insured_losses(losses, deductible, insured_limit):
+def insured_losses(losses, deductible, insurance_limit):
     """
     :param losses: array of ground-up losses
     :param deductible: array of deductible values
-    :param insured_limit: array of insurance limit values
+    :param insurance_limit: array of insurance limit values
 
     Compute insured losses for the given asset and losses, from the point
     of view of the insurance company. For instance:
@@ -1084,13 +1126,14 @@ def insured_losses(losses, deductible, insured_limit):
     assert isinstance(losses, numpy.ndarray), losses
     if not isinstance(deductible, numpy.ndarray):
         deductible = numpy.full_like(losses, deductible)
-    if not isinstance(insured_limit, numpy.ndarray):
-        insured_limit = numpy.full_like(losses, insured_limit)
+    if not isinstance(insurance_limit, numpy.ndarray):
+        insurance_limit = numpy.full_like(losses, insurance_limit)
+    assert (deductible < insurance_limit).all()
     small = losses < deductible
-    big = losses > insured_limit
+    big = losses > insurance_limit
     out = losses - deductible
     out[small] = 0.
-    out[big] = insured_limit[big] - deductible[big]
+    out[big] = insurance_limit[big] - deductible[big]
     return out
 
 
@@ -1104,6 +1147,7 @@ def insurance_losses(asset_df, losses_by_lt, policy_df):
         policy_df.set_index('policy'), on='policy', how='inner')
     for lt in policy_df.loss_type.unique():
         if '+' in lt:
+            # add a key like structural+nonstructural to losses_by_lt
             total_losses(asset_df, losses_by_lt, lt)
     for lt, out in list(losses_by_lt.items()):
         if len(out) == 0:
@@ -1115,8 +1159,8 @@ def insurance_losses(asset_df, losses_by_lt, policy_df):
         new['variance'] = 0.
         j = new.join(adf, on='aid', how='inner')
         if '+' in lt:
-            values = numpy.sum(
-                j['value-' + ltype].to_numpy() for ltype in lt.split('+'))
+            lst = [j['value-' + ltype].to_numpy() for ltype in lt.split('+')]
+            values = numpy.sum(lst, axis=0)  # shape num_values
         else:
             values = j['value-' + lt].to_numpy()
         losses = j.loss.to_numpy()
@@ -1139,13 +1183,13 @@ def total_losses(asset_df, losses_by_lt, kind):
     losses_by_lt[kind] = _agg([losses_by_lt[lt] for lt in ltypes])
 
 
-def insurance_loss_curve(curve, deductible, insured_limit):
+def insurance_loss_curve(curve, deductible, insurance_limit):
     """
     Compute an insured loss ratio curve given a loss ratio curve
 
     :param curve: an array 2 x R (where R is the curve resolution)
     :param float deductible: the deductible limit in fraction form
-    :param float insured_limit: the insured limit in fraction form
+    :param float insurance_limit: the insured limit in fraction form
 
     >>> losses = numpy.array([3, 20, 101])
     >>> poes = numpy.array([0.9, 0.5, 0.1])
@@ -1153,7 +1197,7 @@ def insurance_loss_curve(curve, deductible, insured_limit):
     array([[ 3.        , 20.        ],
            [ 0.85294118,  0.5       ]])
     """
-    losses, poes = curve[:, curve[0] <= insured_limit]
+    losses, poes = curve[:, curve[0] <= insurance_limit]
     limit_poe = interpolate.interp1d(
         *curve, bounds_error=False, fill_value=1)(deductible)
     return numpy.array([
@@ -1349,7 +1393,20 @@ def return_periods(eff_time, num_losses):
     return U32(periods)
 
 
-def losses_by_period(losses, return_periods, num_events=None, eff_time=None):
+def maximum_probable_loss(losses, return_period, eff_time, sorting_idxs=None):
+    """
+    :returns: Maximum Probable Loss at the given return period
+
+    >>> losses = [1000., 0., 2000., 1500., 780., 900., 1700., 0., 100., 200.]
+    >>> maximum_probable_loss(losses, 2000, 10_000)
+    900.0
+    """
+    return losses_by_period(losses, [return_period], len(losses), eff_time,
+                            sorting_idxs)[0]
+
+
+def losses_by_period(losses, return_periods, num_events=None, eff_time=None,
+                     sorting_idxs=None):
     """
     :param losses: simulated losses
     :param return_periods: return periods of interest
@@ -1382,7 +1439,10 @@ def losses_by_period(losses, return_periods, num_events=None, eff_time=None):
             % (num_events, num_losses))
     if eff_time is None:
         eff_time = return_periods[-1]
-    losses = numpy.sort(losses)
+    if sorting_idxs is None:
+        losses = numpy.sort(losses)
+    else:
+        losses = losses[sorting_idxs]
     # num_losses < num_events: just add zeros
     num_zeros = num_events - num_losses
     if num_zeros:
@@ -1433,6 +1493,7 @@ class LossCurvesMapsBuilder(object):
 
 def _agg(loss_dfs, weights=None):
     # average loss DataFrames with fields (eid, aid, variance, loss)
+    # NB: if there are weights the DataFrames are changed!!
     if weights is not None:
         for loss_df, w in zip(loss_dfs, weights):
             loss_df['variance'] *= w
@@ -1476,8 +1537,8 @@ class RiskComputer(dict):
         dic = collections.defaultdict(list)  # lt -> outs
         weights = collections.defaultdict(list)  # lt -> weights
         event = hasattr(haz, 'eid')  # else classical (haz.array)
-        for key, lt in self:
-            rm = self[key, lt]
+        for riskid, lt in self:
+            rm = self[riskid, lt]
             if len(rm.imt_by_lt) == 1:
                 # NB: if `check_risk_ids` raise an error then
                 # this code branch will never run
@@ -1489,7 +1550,7 @@ class RiskComputer(dict):
                 out = rm(lt, self.asset_df, haz, col, rndgen)
             else:  # classical
                 out = rm(lt, self.asset_df, haz.array[self.imtls(imt), 0])
-            weights[lt].append(self.wdic[key, lt])
+            weights[lt].append(self.wdic[riskid, lt])
             dic[lt].append(out)
         out = {}
         for lt in self.minimum_asset_loss:
@@ -1532,6 +1593,10 @@ class RiskComputer(dict):
                    calculation_mode=self.calculation_mode)
         return dic
 
+    def pprint(self):
+        dic = _reduce(self.todict())
+        pprint(dic)
+
 
 # ####################### Consequences ##################################### #
 
