@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2022 GEM Foundation
+# Copyright (C) 2014-2023 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -36,6 +36,9 @@ OQSERVER_ROOT = os.path.dirname(__file__)
 DEBUG = True
 BASE_DIR = os.path.abspath(os.path.dirname(__file__))
 
+WEBUI_PATHPREFIX = os.getenv('WEBUI_PATHPREFIX', '')
+USE_X_FORWARDED_HOST = os.getenv('USE_X_FORWARDED_HOST', False)
+
 TEMPLATES = [
     {
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
@@ -54,8 +57,6 @@ TEMPLATES = [
     },
 ]
 
-STATIC_URL = '/static/'
-
 MEDIA_ROOT = '%(mediaroot)s'
 STATIC_ROOT = '%(staticroot)s'
 
@@ -204,6 +205,8 @@ except ImportError:
         # settings in this file only will be used
         pass
 
+STATIC_URL = '%s/static/' % WEBUI_PATHPREFIX
+
 if LOCKDOWN:
 
     AUTHENTICATION_BACKENDS += (
@@ -247,6 +250,8 @@ if LOCKDOWN:
         },
     ]
 
-    LOGIN_REDIRECT_URL = '/engine'
+    LOGIN_REDIRECT_URL = '%s/engine/' % WEBUI_PATHPREFIX
+    LOGOUT_REDIRECT_URL = '%s/accounts/login/' % WEBUI_PATHPREFIX
+    LOGIN_EXEMPT_URLS = ('%s/accounts/ajax_login/' % WEBUI_PATHPREFIX, )
+    LOGIN_URL = '%s/accounts/login/' % WEBUI_PATHPREFIX
 
-    LOGIN_EXEMPT_URLS = ('/accounts/ajax_login/', )
